Package: xiacf
Type: Package
Title: Nonlinear Dependence and Lead-Lag Analysis via Chatterjee's Xi
Version: 0.6.2
Date: 2026-06-02
Authors@R: person(given = "Yasunori", family = "Watanabe", role = c("aut", "cre"), email = "watanabe.yasunori@outlook.com")
Maintainer: Yasunori Watanabe <watanabe.yasunori@outlook.com>
Description: Computes Chatterjee's non-parametric correlation coefficient for time series data. It extends the original metric to time series analysis by providing the univariate Xi-Autocorrelation Function (Xi-ACF), directional Xi-Cross-Correlation Function (Xi-CCF), and multivariate network evaluation matrices. The package allows users to test for non-linear dependence using Iterative Amplitude Adjusted Fourier Transform (IAAFT) and Multivariate IAAFT (MIAAFT) surrogate data with strict Family-Wise Error Rate ('FWER') control via Max-statistic approaches. Methodologies are based on Chatterjee (2021) <doi:10.1080/01621459.2020.1758115>, surrogate data testing methods by Schreiber and Schmitz (1996) <doi:10.1103/PhysRevLett.77.635>, and local structural identification by Watanabe (2026) <doi:10.2139/ssrn.6829431>.
License: MIT + file LICENSE
Encoding: UTF-8
Imports: dplyr (>= 1.1.4), doFuture, foreach, future, ggplot2 (>=
        4.0.1), patchwork, progressr, Rcpp (>= 1.1.0), stats,
        parallelly
LinkingTo: Rcpp, RcppArmadillo
Suggests: testthat (>= 3.3.2)
Config/testthat/edition: 3
Config/roxygen2/version: 8.0.0
RoxygenNote: 7.3.3
NeedsCompilation: yes
Packaged: 2026-06-02 07:06:24 UTC; yasunori
Author: Yasunori Watanabe [aut, cre]
Repository: CRAN
Date/Publication: 2026-06-02 07:30:02 UTC
Built: R 4.5.2; aarch64-apple-darwin20; 2026-06-02 11:05:28 UTC; unix
Archs: xiacf.so.dSYM

Package: xiacf
Type: Package
Title: Nonlinear Dependence and Lead-Lag Analysis via Chatterjee's Xi
Version: 0.6.2
Date: 2026-06-02
Authors@R: person(given = "Yasunori", family = "Watanabe", role = c("aut", "cre"), email = "watanabe.yasunori@outlook.com")
Maintainer: Yasunori Watanabe <watanabe.yasunori@outlook.com>
Description: Computes Chatterjee's non-parametric correlation coefficient for time series data. It extends the original metric to time series analysis by providing the univariate Xi-Autocorrelation Function (Xi-ACF), directional Xi-Cross-Correlation Function (Xi-CCF), and multivariate network evaluation matrices. The package allows users to test for non-linear dependence using Iterative Amplitude Adjusted Fourier Transform (IAAFT) and Multivariate IAAFT (MIAAFT) surrogate data with strict Family-Wise Error Rate ('FWER') control via Max-statistic approaches. Methodologies are based on Chatterjee (2021) <doi:10.1080/01621459.2020.1758115>, surrogate data testing methods by Schreiber and Schmitz (1996) <doi:10.1103/PhysRevLett.77.635>, and local structural identification by Watanabe (2026) <doi:10.2139/ssrn.6829431>.
License: MIT + file LICENSE
Encoding: UTF-8
Imports: dplyr (>= 1.1.4), doFuture, foreach, future, ggplot2 (>=
        4.0.1), patchwork, progressr, Rcpp (>= 1.1.0), stats,
        parallelly
LinkingTo: Rcpp, RcppArmadillo
Suggests: testthat (>= 3.3.2)
Config/testthat/edition: 3
Config/roxygen2/version: 8.0.0
RoxygenNote: 7.3.3
NeedsCompilation: yes
Packaged: 2026-06-02 07:06:24 UTC; yasunori
Author: Yasunori Watanabe [aut, cre]
Repository: CRAN
Date/Publication: 2026-06-02 07:30:02 UTC
Built: R 4.6.0; aarch64-apple-darwin23; 2026-06-02 11:47:45 UTC; unix
Archs: xiacf.so.dSYM

# Generated by roxygen2: do not edit by hand

S3method(autoplot,xi_acf)
S3method(autoplot,xi_ccf)
S3method(autoplot,xi_matrix)
S3method(print,xi_acf)
S3method(print,xi_ccf)
S3method(print,xi_matrix)
export(autoplot)
export(compute_xi_acf_iaaft)
export(compute_xi_acf_maxstat_cpp)
export(compute_xi_ccf_maxstat_cpp)
export(compute_xi_ccf_miaaft)
export(compute_xi_matrix_maxstat_cpp)
export(compute_xi_matrix_miaaft)
export(extract_xi_acf)
export(extract_xi_ccf)
export(generate_iaaft_surrogate)
export(generate_miaaft_surrogate_cpp)
export(generate_miaaft_surrogates)
export(run_rolling_xi_acf)
export(run_rolling_xi_analysis)
export(run_rolling_xi_ccf)
export(surrogate_iaaft_cpp)
export(surrogate_miaaft_cpp)
export(xi_acf)
export(xi_ccf)
export(xi_coefficient)
export(xi_matrix)
export(xi_test)
importFrom(Rcpp,evalCpp)
importFrom(doFuture,"%dofuture%")
importFrom(doFuture,registerDoFuture)
importFrom(dplyr,bind_rows)
importFrom(foreach,foreach)
importFrom(future,multisession)
importFrom(future,plan)
importFrom(future,sequential)
importFrom(ggplot2,aes)
importFrom(ggplot2,autoplot)
importFrom(ggplot2,coord_cartesian)
importFrom(ggplot2,element_blank)
importFrom(ggplot2,element_rect)
importFrom(ggplot2,element_text)
importFrom(ggplot2,geom_hline)
importFrom(ggplot2,geom_line)
importFrom(ggplot2,geom_point)
importFrom(ggplot2,geom_ribbon)
importFrom(ggplot2,geom_vline)
importFrom(ggplot2,ggplot)
importFrom(ggplot2,guide_legend)
importFrom(ggplot2,guides)
importFrom(ggplot2,labs)
importFrom(ggplot2,scale_color_manual)
importFrom(ggplot2,scale_fill_manual)
importFrom(ggplot2,scale_size_manual)
importFrom(ggplot2,scale_x_continuous)
importFrom(ggplot2,theme)
importFrom(ggplot2,theme_minimal)
importFrom(parallelly,availableCores)
importFrom(patchwork,plot_annotation)
importFrom(patchwork,wrap_plots)
importFrom(progressr,progressor)
importFrom(progressr,with_progress)
importFrom(stats,quantile)
importFrom(stats,var)
importFrom(utils,head)
useDynLib(xiacf, .registration = TRUE)

# xiacf: Nonlinear Dependence and Lead-Lag Analysis via Chatterjee’s Xi

[![CRAN
status](https://www.r-pkg.org/badges/version/xiacf)](https://CRAN.R-project.org/package=xiacf)
[![R-CMD-check](https://github.com/yetanothersu/xiacf/actions/workflows/R-CMD-check.yaml/badge.svg)](https://github.com/yetanothersu/xiacf/actions/workflows/R-CMD-check.yaml)
[![License:
MIT](https://img.shields.io/badge/License-MIT-yellow.svg)](https://opensource.org/licenses/MIT)
[![DOI](https://zenodo.org/badge/DOI/10.5281/zenodo.19247735.svg)](https://doi.org/10.5281/zenodo.19247735)

## Introduction

The **xiacf** package provides a robust framework for detecting complex
non-linear and functional dependence in time series data. Traditional
linear metrics, such as the standard Autocorrelation Function (ACF) and
Cross-Correlation Function (CCF), often fail to detect symmetrical or
purely non-linear relationships.

This package overcomes these limitations by utilizing **Chatterjee’s
Rank Correlation ($\xi$)**, offering both univariate ($\xi$-ACF) and
multivariate ($\xi$-CCF) analysis tools. It features rigorous
statistical hypothesis testing powered by advanced surrogate data
generation algorithms (IAAFT and MIAAFT) and dynamic Family-Wise Error
Rate (FWER) control, all implemented in high-performance C++ using
`RcppArmadillo`.

## Citation

If you use `xiacf` in your research, please cite our latest working
paper detailing the methodology:

> Watanabe, Y. (2026). Differential diagnosis of nonlinearity:
> Integrating the BDS omnibus test with chatterjee’s xi for local
> structural identification. In Social Science Research Network.
> <https://doi.org/10.2139/ssrn.6829431>

## Key Features

- **Non-linear Autocorrelation ($\xi$-ACF):** Detect time-dependent
  structures that standard linear ACF completely misses, such as chaotic
  systems or volatility clustering (e.g., $X_t = |X_{t-1}|$).
- **Directional Cross-Correlation ($\xi$-CCF):** Uncover asymmetrical
  causal pathways. The metric clearly separates causal directions
  (`X leads Y` vs `Y leads X`) and captures contemporaneous effects
  (Lag 0) where traditional Pearson CCF fails.
- **Multivariate Network Matrix (`xi_matrix`):** Simultaneously evaluate
  causal pathways across an entire system of time series to build
  directed non-linear networks.
- **Rigorous Inference & Dual-Family FWER Control:** Utilizes
  phase-randomized surrogate data (IAAFT / MIAAFT) and dynamically
  controls the FWER via Max-Statistic distributions. To overcome the
  **“Lag-0 Masking Effect”**—where MIAAFT perfectly preserves
  contemporaneous linear confounding—`xiacf` uniquely implements a
  **Dual-Family FWER Control**. It completely decouples the significance
  thresholds for pure contemporaneous effects (Lag 0) and temporal
  causal propagation (Lag \> 0), maximizing the statistical power to
  detect delayed causal spillovers.
- **Strict Algorithmic Integrity:** The core C++ engine rigorously
  implements the exact original specifications of Chatterjee’s $\xi$. By
  enforcing uniform random tie-breaking, the package ensures that the
  baseline calculations strictly align with the mathematical definition
  and its asymptotic properties.
- **Tidyverse-ready & Publication-Quality UI:** All functions return
  Tidy data frames. The `autoplot()` methods instantly generate
  beautiful, unified, and publication-ready `ggplot2` visualizations.
- **High-Performance Rolling Analysis:** Fully parallelized (via
  `doFuture`) rolling window functions to capture time-varying
  dependencies smoothly and efficiently.

## Installation

You can install the stable version of xiacf from CRAN with:

``` r
install.packages("xiacf")
```

You can install the development version from
[GitHub](https://github.com/yetanothersu/xiacf) with:

``` r
# install.packages("remotes")
remotes::install_github("yetanothersu/xiacf")
```

## Basic Usage

### 1. Univariate Non-linear ACF ($\xi$-ACF)

Detecting strong non-linear auto-dependence that standard linear ACF
fails to capture.

``` r
### 1. Univariate Non-linear ACF (xi-ACF)

library(xiacf)
library(ggplot2)

set.seed(42)
n <- 300

# Generate a series with V-shaped auto-dependence (mean zero)
# Standard Pearson ACF will miss this, but Xi-ACF will detect it.
A <- numeric(n)
A[1] <- rnorm(1)
for (t in 2:n) {
  # Subtracting 0.8 keeps the series centered around 0
  A[t] <- abs(A[t - 1]) - 0.8 + rnorm(1, sd = 0.2)
}

res_acf <- xi_acf(A, max_lag = 5, n_surr = 249)
#> Warning in check_surrogate_count(n_surr = n_surr, sig_level = sig_level, :
#> Warning: For 5 simultaneous tests at sig_level = 0.05, the empirical
#> distribution of the max-statistic may be unstable with n_surr = 249.
#> Recommended n_surr is at least 399.
print(res_acf)
#> 
#> === Univariate Xi-Autocorrelation Function ===
#> Time series length: 300
#> Max Lag: 5
#> Surrogates (IAAFT): 249
#> Significance Level: 0.05 (FWER controlled)
#> ==============================================
#> Significant Lags:
#>  Lag        Xi Global_Threshold Xi_Excess
#>    1 0.4470805        0.2976711 0.1494094
autoplot(res_acf)
```

<img src="man/figures/README-xi-acf-1.png" alt="" width="100%" />

### 2. Bivariate Non-linear CCF ($\xi$-CCF)

Discovering hidden causal pathways across different time series.

``` r
### 2. Bivariate Non-linear CCF (xi-CCF)

# X is pure noise (mean 0, symmetric)
X <- rnorm(n)
Y <- numeric(n)

# Y is a purely quadratic function of X at lag 2
for (t in 3:n) {
  Y[t] <- X[t - 2]^2 + rnorm(1, sd = 0.2)
}

# Center Y so it contains both negative and positive values.
# Without this, Y is always positive, making abs(Y) purely linear later!
Y <- as.numeric(scale(Y))

res_ccf <- xi_ccf(X, Y, max_lag = 5, n_surr = 249, direction = "both")
#> Warning in xi_ccf(X, Y, max_lag = 5, n_surr = 249, direction = "both"):
#> Warning: For 10 simultaneous tests at sig_level = 0.05, the empirical
#> distribution of the max-statistic may be unstable with n_surr = 249.
#> Recommended n_surr is at least 399.
print(res_ccf)
#> 
#> === Bivariate Xi-Cross-Correlation (CCF) ===
#> Variables: X, Y
#> Time series length: 300
#> Max Lag: 5
#> Direction: both
#> Surrogates (MIAAFT): 249
#> Significance Level: 0.05 (FWER controlled)
#> ============================================
#> Top 5 Strongest Causal Pathways:
#>  Lead_Var Lag_Var Lag        Xi      CCF Global_Threshold Xi_Excess
#>         X       Y   2 0.6388298 0.123921       0.09151595 0.5473138
autoplot(res_ccf)
```

<img src="man/figures/README-xi-ccf-1.png" alt="" width="100%" />

### 3. Multivariate Network Matrix

Analyze an entire system of variables at once.

``` r
### 3. Multivariate Network Matrix & Pathway Extraction

Z <- numeric(n)

# Z depends on the absolute value of Y at lag 1
# Because Y is centered, this creates a true V-shaped non-linear relationship
for (t in 2:n) {
  Z[t] <- abs(Y[t - 1]) + rnorm(1, sd = 0.2)
}
Z <- as.numeric(scale(Z))

df_system <- data.frame(X = X, Y = Y, Z = Z)

# Compute the multivariate Xi-correlogram matrix
res_matrix <- xi_matrix(df_system, max_lag = 3, n_surr = 499)

autoplot(res_matrix)
```

<img src="man/figures/README-xi-matrix-1.png" alt="" width="100%" />

``` r

# Extract it for a detailed bivariate analysis with exact FWER re-evaluation!
ext_ccf <- extract_xi_ccf(res_matrix, var_x = "X", var_y = "Z", direction = "x_leads")
autoplot(ext_ccf)
```

<img src="man/figures/README-xi-matrix-2.png" alt="" width="100%" />

### 4. Rolling Analysis for Dynamic Relationships

Extract time-varying non-linear dependencies. The output is a Tidy data
frame, perfectly structured for custom EDA and visualization.

``` r
### 4. Rolling Analysis for Dynamic Relationships

library(future)
plan(multisession) # or multicore on Linux/macOS

rolling_res <- run_rolling_xi_ccf(
  x = X,
  y = Y,
  window_size = 50,
  step_size = 10,
  max_lag = 3,
  n_surr = 199
)

head(rolling_res)
#>   Window_ID Lead_Var Lag_Var Lag          Xi Global_Threshold Xi_Excess
#> 1         1        x       y   0 -0.03601441        0.2570228 0.0000000
#> 2         1        x       y   1 -0.04625000        0.2185806 0.0000000
#> 3         1        x       y   2  0.54798089        0.2185806 0.3294003
#> 4         1        x       y   3 -0.06657609        0.2185806 0.0000000
#> 5         1        y       x   0 -0.07803121        0.2570228 0.0000000
#> 6         1        y       x   1  0.00125000        0.2185806 0.0000000
```

## References

The theoretical foundation and surrogate data methodologies implemented
in this package are based on the following works:

- **Chatterjee’s $\xi$:** Chatterjee, S. (2021). A new coefficient of
  correlation. *Journal of the American Statistical Association*,
  116(536), 2009-2022. <https://doi.org/10.1080/01621459.2020.1758115>
- **IAAFT / Surrogate Data:** Schreiber, T., & Schmitz, A. (1996).
  Improved surrogate data for nonlinearity tests. *Physical Review
  Letters*, 77(4), 635. <https://doi.org/10.1103/PhysRevLett.77.635>
- **Local Structural Identification (Package Methodology):**
  Watanabe, Y. (2026). Differential diagnosis of nonlinearity:
  Integrating the BDS omnibus test with chatterjee’s xi for local
  structural identification. *SSRN Preprint*.
  <https://doi.org/10.2139/ssrn.6829431>

## License

This project is licensed under the MIT License - see the LICENSE file
for details.

#' @importFrom Rcpp evalCpp
#' @useDynLib xiacf, .registration = TRUE
NULL

#' Compute empirical Xi-ACF and its significance via IAAFT surrogates
#'
#' @param x A numeric vector representing the time series data. Must not contain missing values (NA) or be a constant.
#' @param max_lag An integer specifying the maximum lag to compute. Default is 10.
#' @param n_surr An integer specifying the number of surrogate datasets to generate. Default is 399.
#' @param sig_level A numeric value between 0 and 1 specifying the significance level. Default is 0.05.
#' @param max_iter An integer specifying the maximum iterations for the IAAFT algorithm. Default is 100.
#' @param ... Additional arguments (currently ignored).
#'
#' @return An object of class \code{xi_acf} containing the empirical ACF,
#'   pointwise thresholds, global threshold, and metadata.
#' @export
xi_acf <- function(
    x,
    max_lag = 10,
    n_surr = 399,
    sig_level = 0.05,
    max_iter = 100,
    ...
) {
    # --- Backward Compatibility Check for sig_level ---
    if (sig_level <= 0 || sig_level >= 1) {
        stop("Parameter 'sig_level' must be strictly between 0 and 1.")
    }
    if (sig_level > 0.5) {
        warning(
            sprintf(
                "The interpretation of 'sig_level' has changed from Confidence Level to Significance Level. Your input %g has been automatically converted to %g. Please use sig_level = %g in the future.",
                sig_level,
                1 - sig_level,
                1 - sig_level
            ),
            call. = FALSE
        )
        sig_level <- 1 - sig_level
    }

    if (!is.numeric(x)) {
        stop("Input 'x' must be a numeric vector.")
    }
    if (any(is.na(x))) {
        stop(
            "Input 'x' contains NA values. Please impute or remove them before running xi_acf()."
        )
    }

    n <- length(x)
    if (n < (max_lag + 2)) {
        stop(sprintf(
            "Time series length (%d) is too short for max_lag = %d. Minimum length required is %d.",
            n,
            max_lag,
            max_lag + 2
        ))
    }
    if (stats::var(x) == 0) {
        stop(
            "Input 'x' has zero variance (constant series). Xi coefficient cannot be computed."
        )
    }

    check_surrogate_count(
        n_surr = n_surr,
        sig_level = sig_level,
        num_tests = max_lag
    )

    acf_vals <- as.numeric(stats::acf(x, lag.max = max_lag, plot = FALSE)$acf[
        -1
    ])

    cpp_res <- compute_xi_acf_maxstat_cpp(
        x = as.numeric(x),
        max_lag = as.integer(max_lag),
        n_surr = as.integer(n_surr),
        max_iter = as.integer(max_iter)
    )

    pointwise_threshold <- stats::quantile(
        cpp_res$pointwise_dist,
        probs = 1 - sig_level,
        names = FALSE,
        na.rm = TRUE
    )
    global_threshold <- stats::quantile(
        cpp_res$max_statistic_dist,
        probs = 1 - sig_level,
        names = FALSE,
        na.rm = TRUE
    )

    acf_ci <- stats::qnorm((1 + (1 - sig_level)) / 2) / sqrt(n)

    df_res <- data.frame(
        Lag = 1:max_lag,
        ACF = acf_vals,
        Xi = cpp_res$xi_empirical,
        Pointwise_Threshold = pointwise_threshold,
        Global_Threshold = rep(global_threshold, max_lag),
        ACF_CI = rep(acf_ci, max_lag)
    )

    df_res$Xi_Excess <- pmax(0, df_res$Xi - df_res$Global_Threshold)

    structure(
        list(
            data = df_res,
            n = n,
            max_lag = max_lag,
            n_surr = n_surr,
            sig_level = sig_level
        ),
        class = "xi_acf"
    )
}

#' @rdname xi_acf
#' @export
print.xi_acf <- function(x, ...) {
    cat("\n=== Univariate Xi-Autocorrelation Function ===\n")
    cat(sprintf("Time series length: %d\n", x$n))
    cat(sprintf("Max Lag: %d\n", x$max_lag))
    cat(sprintf("Surrogates (IAAFT): %d\n", x$n_surr))
    cat(sprintf("Significance Level: %g (FWER controlled)\n", x$sig_level))
    cat("==============================================\n")

    sig_data <- x$data[
        x$data$Xi_Excess > 0,
        c("Lag", "Xi", "Global_Threshold", "Xi_Excess")
    ]

    if (nrow(sig_data) == 0) {
        cat(
            "No significant autocorrelations found above the global threshold.\n"
        )
    } else {
        cat("Significant Lags:\n")
        print(sig_data, row.names = FALSE)
    }
    cat("\n")
    invisible(x)
}

test_that("xi_acf function works and returns correct structure", {
  set.seed(42)
  x <- rnorm(100)
  max_lag <- 10
  res <- suppressWarnings(xiacf::xi_acf(x, max_lag = max_lag, n_surr = 20))

  expect_s3_class(res, "xi_acf")
  expected_cols <- c(
    "Lag",
    "ACF",
    "Xi",
    "Global_Threshold",
    "ACF_CI",
    "Xi_Excess"
  )
  expect_true(all(expected_cols %in% names(res$data)))
})

test_that("xi_acf handles input errors gracefully", {
  expect_error(xiacf::xi_acf(c(1, 2, 3)), "too short")
  expect_error(xiacf::xi_acf(rep(1, 100)), "zero variance")
  expect_error(xiacf::xi_acf(c(rnorm(10), NA), max_lag = 5), "NA values")
})

test_that("sig_level dynamically adjusts thresholds and CIs", {
  set.seed(42)
  x <- rnorm(100)

  res_05 <- suppressWarnings(xiacf::xi_acf(
    x,
    max_lag = 5,
    n_surr = 100,
    sig_level = 0.05
  ))
  res_01 <- suppressWarnings(xiacf::xi_acf(
    x,
    max_lag = 5,
    n_surr = 100,
    sig_level = 0.01
  ))

  expect_gt(res_01$data$ACF_CI[1], res_05$data$ACF_CI[1])
  expect_gt(
    mean(res_01$data$Global_Threshold),
    mean(res_05$data$Global_Threshold)
  )
})

#' Bivariate Xi-Cross-Correlation Function
#'
#' Computes the directional Chatterjee's Xi coefficient between two time series
#' across multiple lags, with Family-Wise Error Rate (FWER) control.
#'
#' @param x A numeric vector representing the first time series.
#' @param y A numeric vector representing the second time series.
#' @param max_lag An integer specifying the maximum lag to compute. Default is 20.
#' @param n_surr An integer specifying the number of MIAAFT surrogate datasets. Default is 399.
#' @param sig_level A numeric value specifying the significance level (FWER). Default is 0.05.
#' @param max_iter An integer specifying the maximum iterations for the MIAAFT algorithm. Default is 100.
#' @param direction A character string specifying the testing direction.
#'        "both" computes X->Y and Y->X. "x_leads" computes only X->Y. Default is "both".
#' @param ... Additional arguments.
#'
#' @return An S3 object of class \code{xi_ccf}.
#' @export
xi_ccf <- function(
    x,
    y,
    max_lag = 20,
    n_surr = 399,
    sig_level = 0.05,
    max_iter = 100,
    direction = c("both", "x_leads"),
    ...
) {
    # Parse direction argument
    direction <- match.arg(direction)
    both_directions <- (direction == "both")

    # Extract variable names for plotting
    x_name <- deparse(substitute(x))
    y_name <- deparse(substitute(y))

    # --- Backward Compatibility Check for sig_level ---
    if (sig_level <= 0 || sig_level >= 1) {
        stop("Parameter 'sig_level' must be strictly between 0 and 1.")
    }
    if (sig_level > 0.5) {
        warning(
            sprintf(
                "The interpretation of 'sig_level' has changed from Confidence Level to Significance Level. Your input %g has been automatically converted to %g. Please use sig_level = %g in the future.",
                sig_level,
                1 - sig_level,
                1 - sig_level
            ),
            call. = FALSE
        )
        sig_level <- 1 - sig_level
    }

    # Input validation
    if (length(x) != length(y)) {
        stop("Time series 'x' and 'y' must have the same length.")
    }
    if (any(is.na(x)) || any(is.na(y))) {
        stop(
            "Input time series contain NA values. Please handle them before running xi_ccf()."
        )
    }
    if (stats::var(x) == 0 || stats::var(y) == 0) {
        stop("Time series 'x' and 'y' must have non-zero variance.")
    }

    n <- length(x)
    if (n < max_lag + 2) {
        stop(sprintf(
            "Time series length (%d) is too short for max_lag = %d.",
            n,
            max_lag
        ))
    }

    # Dynamic calculation of family size for Dual-Family FWER control
    num_tests_lag0 <- if (both_directions) 2 else 1
    num_tests_lagged <- if (both_directions) 2 * max_lag else max_lag

    # Use the maximum family size between Lag 0 and Lag > 0 for surrogate stability check
    num_tests <- max(num_tests_lag0, num_tests_lagged)

    # Surrogate count warning logic
    min_required <- ceiling(1 / sig_level) - 1
    if (n_surr < min_required) {
        stop(sprintf(
            "Error: n_surr = %d is too small to calculate the %d%% threshold. Minimum required is %d.",
            n_surr,
            as.integer((1 - sig_level) * 100),
            min_required
        ))
    }
    recommended <- ceiling(num_tests / sig_level)
    recommended <- max(399, recommended) # Enforce typical non-parametric baseline

    if (n_surr < recommended) {
        warning(sprintf(
            "Warning: For %d simultaneous tests at sig_level = %g, the empirical distribution of the max-statistic may be unstable with n_surr = %d. Recommended n_surr is at least %d.",
            num_tests,
            sig_level,
            n_surr,
            recommended
        ))
    }

    # Execute C++ engine
    cpp_res <- compute_xi_ccf_maxstat_cpp(
        x = as.numeric(x),
        y = as.numeric(y),
        max_lag = as.integer(max_lag),
        n_surr = as.integer(n_surr),
        max_iter = as.integer(max_iter),
        both_directions = both_directions
    )

    # Calculate two independent global thresholds
    threshold_lag0 <- stats::quantile(
        cpp_res$max_dist_lag0,
        probs = 1 - sig_level,
        names = FALSE,
        na.rm = TRUE
    )
    threshold_lagged <- stats::quantile(
        cpp_res$max_dist_lagged,
        probs = 1 - sig_level,
        names = FALSE,
        na.rm = TRUE
    )

    # =========================================================================
    # Compute linear CCF and Confidence Interval (CI)
    # Note: In standard stats::ccf(), negative lags represent X->Y,
    # and positive lags represent Y->X.
    # =========================================================================
    ccf_obj <- stats::ccf(
        as.numeric(x),
        as.numeric(y),
        lag.max = max_lag,
        plot = FALSE
    )

    # X leads Y (lag 0 to max_lag): extract and reverse the negative lag side of CCF
    ccf_x_leads <- rev(as.vector(ccf_obj$acf[1:(max_lag + 1)]))
    # Y leads X (lag 0 to max_lag): extract the positive lag side of CCF
    ccf_y_leads <- as.vector(ccf_obj$acf[(max_lag + 1):(2 * max_lag + 1)])

    # Asymptotic confidence interval for CCF
    ccf_ci <- stats::qnorm(1 - sig_level / 2) / sqrt(n)

    # Format output into a tidy data frame
    lag_vec <- 0:max_lag

    if (both_directions) {
        # Bind both directions vertically (Long format)
        res_df <- data.frame(
            Lead_Var = c(
                rep(x_name, length(lag_vec)),
                rep(y_name, length(lag_vec))
            ),
            Lag_Var = c(
                rep(y_name, length(lag_vec)),
                rep(x_name, length(lag_vec))
            ),
            Lag = c(lag_vec, lag_vec),
            Xi = c(cpp_res$xi_emp_x_leads, cpp_res$xi_emp_y_leads),
            CCF = c(ccf_x_leads, ccf_y_leads),
            CCF_CI = rep(ccf_ci, 2 * length(lag_vec)),
            stringsAsFactors = FALSE
        )
    } else {
        # X leads Y only
        res_df <- data.frame(
            Lead_Var = rep(x_name, length(lag_vec)),
            Lag_Var = rep(y_name, length(lag_vec)),
            Lag = lag_vec,
            Xi = cpp_res$xi_emp_x_leads,
            CCF = ccf_x_leads,
            CCF_CI = rep(ccf_ci, length(lag_vec)),
            stringsAsFactors = FALSE
        )
    }

    # Dynamic threshold assignment based on Lag
    is_lag_zero <- res_df$Lag == 0
    res_df$Global_Threshold <- NA_real_
    res_df$Global_Threshold[is_lag_zero] <- threshold_lag0
    res_df$Global_Threshold[!is_lag_zero] <- threshold_lagged

    # Calculate Xi_Excess
    res_df$Xi_Excess <- pmax(0, res_df$Xi - res_df$Global_Threshold)

    # Construct output object
    out <- list(
        data = res_df,
        n = n,
        max_lag = max_lag,
        n_surr = n_surr,
        sig_level = sig_level,
        direction = direction,
        x_name = x_name,
        y_name = y_name
    )

    class(out) <- "xi_ccf"
    return(out)
}

#' Print method for xi_ccf
#' @param x An object of class \code{xi_ccf}.
#' @param ... Additional arguments passed to print.
#' @return The original object \code{x} invisibly.
#' @importFrom utils head
#' @export
print.xi_ccf <- function(x, ...) {
    cat("\n=== Bivariate Xi-Cross-Correlation (CCF) ===\n")
    cat(sprintf("Variables: %s, %s\n", x$x_name, x$y_name))
    cat(sprintf("Time series length: %d\n", x$n))
    cat(sprintf("Max Lag: %d\n", x$max_lag))
    cat(sprintf("Direction: %s\n", x$direction))
    cat(sprintf("Surrogates (MIAAFT): %d\n", x$n_surr))
    cat(sprintf("Significance Level: %g (FWER controlled)\n", x$sig_level))
    cat("============================================\n")

    # Safely extract significant paths
    sig_data <- x$data[
        which(x$data$Xi_Excess > 0),
        c(
            "Lead_Var",
            "Lag_Var",
            "Lag",
            "Xi",
            "CCF",
            "Global_Threshold",
            "Xi_Excess"
        )
    ]

    if (nrow(sig_data) == 0) {
        cat(
            "No significant directional dependencies found above the global threshold.\n"
        )
    } else {
        cat("Top 5 Strongest Causal Pathways:\n")
        print(
            utils::head(sig_data[order(-sig_data$Xi_Excess), ], 5),
            row.names = FALSE
        )
    }
    cat("\n")
    invisible(x)
}

# tests/testthat/test-xi_ccf.R

# tests/testthat/test-xi_ccf.R

test_that("xi_ccf computes correctly and returns proper structure", {
    set.seed(42)
    x <- rnorm(100)
    y <- rnorm(100)
    max_lag <- 3

    # Test for direction = "both"
    res_both <- suppressWarnings(xi_ccf(
        x,
        y,
        max_lag = max_lag,
        n_surr = 20,
        direction = "both"
    ))
    expect_s3_class(res_both, "xi_ccf")
    expect_equal(nrow(res_both$data), 2 * (max_lag + 1)) # 2*(max_lag+1) rows because it includes Lag 0

    # Check for mandatory columns in tidy format
    expected_cols <- c(
        "Lead_Var",
        "Lag_Var",
        "Lag",
        "Xi",
        "CCF",
        "Global_Threshold",
        "Xi_Excess",
        "CCF_CI"
    )
    expect_true(all(expected_cols %in% colnames(res_both$data)))

    # Test for direction = "x_leads"
    res_x <- suppressWarnings(xi_ccf(
        x,
        y,
        max_lag = max_lag,
        n_surr = 20,
        direction = "x_leads"
    ))
    expect_equal(nrow(res_x$data), max_lag + 1)
    expect_true(all(res_x$data$Lead_Var == "x" & res_x$data$Lag_Var == "y"))
})

test_that("xi_ccf handles invalid inputs correctly", {
    set.seed(42)
    x <- rnorm(100)
    y <- rnorm(100)

    # Different lengths
    expect_error(xi_ccf(x[1:90], y), "same length")
    # Contains NA
    expect_error(xi_ccf(c(x[1:99], NA), y), "NA values")
    # Zero variance
    expect_error(xi_ccf(rep(1, 100), y), "non-zero variance")
})

test_that("Dual-Family FWER Control separates Lag 0 and Lag > 0 thresholds", {
    set.seed(42)
    x <- rnorm(100)
    y <- rnorm(100)

    # Suppress warnings for low surrogate count to speed up tests
    res <- suppressWarnings(xiacf::xi_ccf(
        x,
        y,
        max_lag = 3,
        n_surr = 100,
        sig_level = 0.05
    ))

    df <- res$data

    thresh_lag0 <- df$Global_Threshold[df$Lag == 0][1]
    thresh_lagged <- df$Global_Threshold[df$Lag > 0][1]

    expect_false(is.na(thresh_lag0))
    expect_false(is.na(thresh_lagged))

    # Because MIAAFT preserves Lag 0 structures, the threshold for Lag 0
    # should be calculated independently and generally differs from Lag > 0.
    expect_true(thresh_lag0 != thresh_lagged)
})

#' Rolling Multivariate Xi-CCF Analysis
#'
#' Performs a rolling window analysis using Chatterjee's Xi cross-correlation to assess
#' the time-varying non-linear lead-lag relationship between two time series.
#'
#' @param x A numeric vector representing the first time series (predictor/lead candidate).
#' @param y A numeric vector representing the second time series (response/lag candidate).
#' @param time_index Optional vector of timestamps (e.g., Date, POSIXct) corresponding to x and y.
#' @param window_size An integer specifying the size of the rolling window.
#' @param step_size An integer specifying the step size by which the window is shifted. Default is 1.
#' @param max_lag An integer specifying the maximum positive lag to compute.
#' @param n_surr An integer specifying the number of MIAAFT surrogate datasets for the null hypothesis test.
#' @param bidirectional Logical. If TRUE (default), computes both "X leads Y" and "Y leads X".
#' @param sig_level A numeric value specifying the significance level for the confidence intervals. Default is 0.95.
#' @param n_cores An integer specifying the number of cores for parallel execution. If \code{NULL}, runs sequentially.
#' @param save_dir A character string specifying the directory path to save intermediate window results as RDS files. If \code{NULL} (default), results are not saved to disk.
#'
#' @return A \code{data.frame} containing the rolling window results in a tidy long-format.
#'
#' @importFrom foreach foreach
#' @importFrom doFuture registerDoFuture %dofuture%
#' @importFrom future plan multisession sequential
#' @importFrom progressr progressor with_progress
#' @importFrom dplyr bind_rows
#' @importFrom stats quantile sd ccf qnorm na.pass
#' @export
run_rolling_xi_ccf <- function(
    x,
    y,
    time_index = NULL,
    window_size,
    step_size = 1,
    max_lag = 20,
    n_surr = 100,
    bidirectional = TRUE,
    sig_level = 0.95,
    n_cores = NULL,
    save_dir = NULL
) {
    # --- 1. Robust Input Validation ---
    if (length(x) != length(y)) {
        stop("Time series 'x' and 'y' must have the exact same length.")
    }
    n <- length(x)
    if (window_size > n || window_size <= 2 * max_lag + 5) {
        stop(
            "Invalid window_size. Must be <= length(x) and reasonably larger than max_lag."
        )
    }
    if (!is.null(time_index)) {
        if (length(time_index) != n) {
            stop("time_index must have the exact same length as x and y.")
        }
    }

    # Calculate starting indices for each window
    start_indices <- seq(1, n - window_size + 1, by = step_size)
    n_windows <- length(start_indices)

    # --- 2. Setup Checkpointing ---
    completed_windows <- integer(0)
    if (!is.null(save_dir)) {
        if (!dir.exists(save_dir)) {
            dir.create(save_dir, recursive = TRUE)
        }
        existing_files <- list.files(
            save_dir,
            pattern = "^window_.*\\.rds$",
            full.names = TRUE
        )
        if (length(existing_files) > 0) {
            # Extract window IDs from filenames
            completed_windows <- as.integer(gsub(
                ".*window_([0-9]+)\\.rds",
                "\\1",
                existing_files
            ))
            message(sprintf(
                "Found %d completed windows in %s. Resuming...",
                length(completed_windows),
                save_dir
            ))
        }
    }

    # Identify windows that still need to be processed
    windows_to_process <- setdiff(1:n_windows, completed_windows)

    # If all windows are already processed, load and return them
    if (length(windows_to_process) == 0) {
        message("All windows are already processed. Loading from disk...")
        all_files <- list.files(
            save_dir,
            pattern = "^window_.*\\.rds$",
            full.names = TRUE
        )
        res_list <- lapply(all_files, readRDS)
        return(dplyr::bind_rows(res_list))
    }

    # --- 3. Parallel Backend Setup ---
    old_opts <- options(doFuture.rng.onMisuse = "ignore")
    old_plan <- future::plan()

    on.exit(
        {
            options(old_opts)
            future::plan(old_plan)
        },
        add = TRUE
    )

    doFuture::registerDoFuture()
    if (!is.null(n_cores) && n_cores > 1) {
        future::plan(future::multisession, workers = n_cores)
    } else {
        future::plan(future::sequential)
    }

    # --- 4. Main Rolling Execution ---
    # Define a wrapper for progress reporting
    run_with_progress <- function() {
        p <- progressr::progressor(steps = length(windows_to_process))

        results_list <- foreach::foreach(
            i = windows_to_process,
            .errorhandling = 'pass'
        ) %dofuture%
            {
                p() # Update progress bar
                idx_start <- start_indices[i]
                idx_end <- idx_start + window_size - 1

                x_window <- x[idx_start:idx_end]
                y_window <- y[idx_start:idx_end]

                # If variance is zero, return NULL to skip
                if (stats::sd(x_window) == 0 || stats::sd(y_window) == 0) {
                    return(NULL)
                }

                # Standard CCF Confidence Interval
                ccf_ci <- stats::qnorm((1 + sig_level) / 2) / sqrt(window_size)

                # Call C++ Engine
                res <- compute_xi_ccf_miaaft(
                    x_window,
                    y_window,
                    max_lag,
                    n_surr
                )

                # Helper to calculate 95% threshold
                calc_threshold <- function(surr_matrix) {
                    if (n_surr == 0) {
                        return(rep(NA, length(res$lags)))
                    }
                    apply(surr_matrix, 1, function(r) {
                        stats::quantile(r, sig_level, na.rm = TRUE)
                    })
                }

                # --- Build Long-Format DataFrame for the Window ---

                # (A) Forward: X leads Y
                ccf_fwd_full <- stats::ccf(
                    y_window,
                    x_window,
                    lag.max = max_lag,
                    plot = FALSE,
                    na.action = stats::na.pass
                )
                ccf_fwd <- as.numeric(ccf_fwd_full$acf[
                    (max_lag + 1):(2 * max_lag + 1)
                ])

                df_fwd <- data.frame(
                    Direction = "X leads Y",
                    Lag = as.numeric(res$lags),
                    CCF = ccf_fwd,
                    Xi = as.numeric(res$xi_original_forward),
                    Xi_Threshold = calc_threshold(res$xi_surrogates_forward),
                    CCF_CI = ccf_ci
                )

                # (B) Backward: Y leads X
                if (bidirectional) {
                    ccf_bwd_full <- stats::ccf(
                        x_window,
                        y_window,
                        lag.max = max_lag,
                        plot = FALSE,
                        na.action = stats::na.pass
                    )
                    ccf_bwd <- as.numeric(ccf_bwd_full$acf[
                        (max_lag + 1):(2 * max_lag + 1)
                    ])

                    df_bwd <- data.frame(
                        Direction = "Y leads X",
                        Lag = as.numeric(res$lags),
                        CCF = ccf_bwd,
                        Xi = as.numeric(res$xi_original_backward),
                        Xi_Threshold = calc_threshold(
                            res$xi_surrogates_backward
                        ),
                        CCF_CI = ccf_ci
                    )
                    df_window <- rbind(df_fwd, df_bwd)
                } else {
                    df_window <- df_fwd
                }

                # Add Excess Xi and Window Metadata
                df_window$Xi_Excess <- pmax(
                    0,
                    df_window$Xi - df_window$Xi_Threshold
                )
                df_window$Window_ID <- i
                df_window$Window_Start_Idx <- idx_start
                df_window$Window_End_Idx <- idx_end

                # Map the actual timestamps if time_index was provided
                if (!is.null(time_index)) {
                    df_window$Window_Start_Time <- time_index[idx_start]
                    df_window$Window_End_Time <- time_index[idx_end]
                }

                # Save checkpoint
                if (!is.null(save_dir)) {
                    out_file <- file.path(
                        save_dir,
                        sprintf("window_%06d.rds", i)
                    )
                    saveRDS(df_window, file = out_file)
                }

                return(df_window)
            }
        return(results_list)
    }

    # Execute with progress bar
    new_results_list <- progressr::with_progress(run_with_progress())

    # --- 5. Final Result Compilation ---
    # Filter out potential errors/NULLs from parallel execution
    new_results_list <- Filter(function(x) is.data.frame(x), new_results_list)
    final_df <- dplyr::bind_rows(new_results_list)

    # If resumed from disk, combine newly calculated results with old ones
    if (length(completed_windows) > 0 && !is.null(save_dir)) {
        old_files <- file.path(
            save_dir,
            sprintf("window_%06d.rds", completed_windows)
        )
        old_results_list <- lapply(old_files, readRDS)
        old_df <- dplyr::bind_rows(old_results_list)
        final_df <- dplyr::bind_rows(old_df, final_df)
    }

    # Sort correctly taking Direction into account
    if (nrow(final_df) > 0) {
        final_df <- final_df[
            order(final_df$Window_ID, final_df$Direction, final_df$Lag),
        ]
    }

    return(final_df)
}

test_that("run_rolling_xi_acf works and saves intermediate results", {
    set.seed(42)
    x_test <- rnorm(100)
    tmp_dir <- tempfile("xi_rolling_test")
    dir.create(tmp_dir)
    on.exit(unlink(tmp_dir, recursive = TRUE))

    res_initial <- suppressWarnings(run_rolling_xi_acf(
        x = x_test,
        window_size = 80,
        step_size = 10,
        max_lag = 2,
        n_surr = 20,
        save_dir = tmp_dir
    ))
    expect_length(list.files(tmp_dir, pattern = "\\.rds$"), 3)
    expect_s3_class(res_initial, "data.frame")
    expect_true("Global_Threshold" %in% names(res_initial))
})

test_that("run_rolling_xi_ccf works sequentially", {
    set.seed(42)
    x <- rnorm(100)
    y <- rnorm(100)
    window_size <- 50
    step_size <- 10
    max_lag <- 2

    res <- suppressWarnings(run_rolling_xi_ccf(
        x = x,
        y = y,
        window_size = window_size,
        step_size = step_size,
        max_lag = max_lag,
        n_surr = 20
    ))

    expect_s3_class(res, "data.frame")
    expected_windows <- length(seq(1, 100 - window_size + 1, by = step_size))
    expected_rows_per_window <- 2 * (max_lag + 1) # Lag 0起点の双方向
    expect_equal(nrow(res), expected_windows * expected_rows_per_window)
})

test_that("Rolling functions strictly preserve Window_ID for traceability", {
    set.seed(42)
    x <- rnorm(100)
    y <- rnorm(100)

    # Check univariate rolling
    res_acf <- suppressWarnings(xiacf::run_rolling_xi_acf(
        x,
        window_size = 50,
        step_size = 10,
        max_lag = 2,
        n_surr = 50
    ))
    expect_true("Window_ID" %in% colnames(res_acf))
    expect_true(is.numeric(res_acf$Window_ID))
    # Starting at 1, step 10 -> indices 1, 11, 21, 31, 41, 51 (Total 6 windows)
    expect_equal(length(unique(res_acf$Window_ID)), 6)

    # Check bivariate rolling
    res_ccf <- suppressWarnings(xiacf::run_rolling_xi_ccf(
        x,
        y,
        window_size = 50,
        step_size = 10,
        max_lag = 2,
        n_surr = 50
    ))
    expect_true("Window_ID" %in% colnames(res_ccf))
    expect_true(is.numeric(res_ccf$Window_ID))
})

#' Plot Directional Xi-CCF Comparison
#'
#' Visualizes the comparison between the standard linear Cross-Correlation Function (CCF)
#' and the non-linear Chatterjee's Xi cross-correlation across two directions (X leads Y, Y leads X).
#'
#' @param object An object of class \code{"xi_ccf"}.
#' @param ... Additional arguments passed to other methods.
#' @return A \code{ggplot} object representing the directional cross-correlogram.
#' @importFrom ggplot2 autoplot ggplot aes geom_hline geom_ribbon geom_line geom_point guides guide_legend
#' @importFrom ggplot2 scale_color_manual scale_fill_manual scale_linetype_manual coord_cartesian facet_wrap
#' @importFrom ggplot2 labs theme_minimal theme element_text element_rect
#' @importFrom latex2exp TeX
#' @importFrom stats setNames
#' @method autoplot xi_ccf
#' @export
autoplot.xi_ccf <- function(object, ...) {
    if (!requireNamespace("ggplot2", quietly = TRUE)) {
        stop("Package 'ggplot2' is required.")
    }
    if (!requireNamespace("latex2exp", quietly = TRUE)) {
        stop("Package 'latex2exp' is required.")
    }

    df <- object$data
    ccf_ci <- df$CCF_CI[1]
    sig_pct <- object$sig_level * 100

    ccf_ci_label <- paste0("CCF ", sig_pct, "% CI")
    xi_thresh_label <- paste0("Xi ", sig_pct, "% Threshold")

    p <- ggplot(df, aes(x = Lag)) +
        geom_hline(yintercept = 0, color = "gray50", linewidth = 0.3) +
        geom_hline(
            data = data.frame(y_vals = c(ccf_ci, -ccf_ci)),
            aes(yintercept = y_vals, linetype = ccf_ci_label),
            color = "blue",
            linewidth = 0.4,
            alpha = 0.6,
            na.rm = TRUE
        )

    if (!all(is.na(df$Xi_Threshold))) {
        p <- p +
            geom_ribbon(
                aes(ymin = 0, ymax = Xi_Threshold, fill = xi_thresh_label),
                alpha = 0.2
            )
    }

    p <- p +
        geom_line(
            aes(y = CCF, color = "Standard CCF (Linear)"),
            linewidth = 0.6,
            linetype = "dashed",
            na.rm = TRUE
        ) +
        geom_point(
            aes(y = CCF, color = "Standard CCF (Linear)"),
            shape = 16,
            size = 3,
            na.rm = TRUE
        ) +
        geom_line(
            aes(y = Xi, color = "Chatterjee's Xi (Non-linear)"),
            linewidth = 0.8
        ) +

        # Highlight significant points (shape = 24)
        geom_point(
            aes(y = Xi, color = "Chatterjee's Xi (Non-linear)"),
            fill = ifelse(df$Xi_Excess > 0, "firebrick", "white"),
            shape = 24,
            size = ifelse(df$Xi_Excess > 0, 3, 1.5)
        ) +
        facet_wrap(~Direction, ncol = 1) +

        scale_fill_manual(
            name = "Significance",
            values = setNames("gray50", xi_thresh_label),
            labels = TeX(c(paste0(r"($\xi$-CCF )", sig_pct, r"(% Threshold)")))
        ) +
        scale_linetype_manual(
            name = "Significance",
            values = setNames("dotted", ccf_ci_label)
        ) +
        scale_color_manual(
            name = "Correlation Measure",
            breaks = c("Standard CCF (Linear)", "Chatterjee's Xi (Non-linear)"),
            values = c(
                "Standard CCF (Linear)" = "steelblue",
                "Chatterjee's Xi (Non-linear)" = "firebrick"
            ),
            labels = c(
                "Standard CCF (Linear)",
                TeX(r"($\xi$-CCF (Non-linear))")
            )
        ) +
        guides(
            color = guide_legend(
                override.aes = list(
                    shape = c(16, 24),
                    fill = c(NA, "firebrick"),
                    size = 3
                )
            )
        ) +
        labs(
            title = latex2exp::TeX(r"($\xi$-CCF Correlogram)"),
            subtitle = paste0(
                "Linear vs Non-linear Cross-Dependence (n = ",
                object$n,
                ", surrogates = ",
                object$n_surr,
                ")"
            ),
            y = "Correlation Coefficient",
            x = "Lag"
        ) +
        theme_minimal() +
        theme(
            legend.position = "bottom",
            legend.box = "vertical",
            legend.title = element_text(size = 9, face = "bold"),
            plot.title = element_text(face = "bold"),
            strip.text = element_text(
                size = 11,
                face = "bold",
                color = "white"
            ),
            strip.background = element_rect(fill = "gray40", color = NA)
        )

    max_val <- max(c(df$CCF, df$Xi, df$Xi_Threshold, ccf_ci), na.rm = TRUE)
    min_val <- min(c(df$CCF, df$Xi, -ccf_ci, 0), na.rm = TRUE)
    y_margin <- (max_val - min_val) * 0.1
    p <- p +
        coord_cartesian(
            ylim = c(
                max(min_val - y_margin, -1.05),
                min(max_val + y_margin, 1.05)
            )
        ) +
        scale_x_continuous(breaks = function(x) {
            seq(
                ceiling(x[1]),
                floor(x[2]),
                by = max(1, floor((x[2] - x[1]) / 10))
            )
        })
    return(p)
}

#' Plot Multivariate Xi-Correlogram Matrix
#'
#' Visualizes the result of a multivariate Xi-matrix analysis using a facet grid.
#' Rows represent the leading (predictor) variable, and columns represent the lagging (response) variable.
#'
#' @param object An object of class \code{xi_matrix}.
#' @param ... Additional arguments (currently ignored).
#'
#' @return A \code{ggplot} object.
#' @import ggplot2
#' @export
autoplot.xi_matrix <- function(object, ...) {
    df <- object$data
    sig_pct <- object$sig_level * 100

    # 有意性のフラグを作成（Xi_Excess が 0 より大きいか）
    df$Significant <- df$Xi_Excess > 0

    p <- ggplot2::ggplot(df, ggplot2::aes(x = Lag, y = Xi)) +
        # Zero line
        ggplot2::geom_hline(yintercept = 0, color = "gray50", linewidth = 0.3) +

        # 1. Xi Surrogate Threshold (Gray ribbon)
        ggplot2::geom_ribbon(
            ggplot2::aes(ymin = 0, ymax = Xi_Threshold),
            fill = "gray50",
            alpha = 0.2
        ) +

        # 2. Chatterjee's Xi (Line)
        ggplot2::geom_line(
            color = "firebrick",
            linewidth = 0.8
        ) +

        # 3. Points (Highlight significant ones)
        # shape = 24 は枠線(color)と中身(fill)の色を個別に設定できる三角です
        ggplot2::geom_point(
            ggplot2::aes(fill = Significant, size = Significant),
            color = "firebrick",
            shape = 24
        ) +

        # 有意かどうかに応じて、塗りつぶしの色とサイズを動的に変更
        ggplot2::scale_fill_manual(
            values = c("FALSE" = "white", "TRUE" = "firebrick")
        ) +
        ggplot2::scale_size_manual(
            values = c("FALSE" = 1.5, "TRUE" = 3.0) # 有意な点は2倍の大きさに！
        ) +

        # Create the N x N Matrix Grid
        ggplot2::facet_grid(Lead_Var ~ Lag_Var) +

        # Labels and Theme
        ggplot2::labs(
            title = "Multivariate Xi-Correlogram Matrix",
            subtitle = sprintf(
                "Rows: Lead (Predictor)  |  Columns: Lag (Response)\nGray ribbon: %g%% MIAAFT Threshold",
                sig_pct
            ),
            x = "Lag",
            y = "Chatterjee's Xi"
        ) +
        ggplot2::theme_minimal() +
        ggplot2::theme(
            strip.background = ggplot2::element_rect(
                fill = "gray40",
                color = NA
            ),
            strip.text = ggplot2::element_text(
                face = "bold",
                size = 11,
                color = "white"
            ),
            panel.border = ggplot2::element_rect(color = "gray80", fill = NA),
            panel.grid.minor = ggplot2::element_blank(),
            legend.position = "none" # 凡例は不要なので消す
        )

    # --- Dynamic Y-axis Zoom ---
    max_val <- max(c(df$Xi, df$Xi_Threshold), na.rm = TRUE)
    min_val <- min(c(df$Xi, 0), na.rm = TRUE)

    y_margin <- (max_val - min_val) * 0.1
    min_y <- min_val - y_margin
    max_y <- max_val + y_margin

    max_y <- min(max_y, 1.05)
    min_y <- max(min_y, -0.1)

    p <- p +
        ggplot2::coord_cartesian(ylim = c(min_y, max_y)) +
        ggplot2::scale_x_continuous(breaks = function(x) {
            seq(
                ceiling(x[1]),
                floor(x[2]),
                by = max(1, floor((x[2] - x[1]) / 10))
            )
        })

    return(p)
}

#' Validate Number of Surrogates for Max-Statistic FWER Control
#'
#' This is an internal helper function to ensure that the user has provided
#' a sufficient number of surrogate datasets to estimate the specified
#' significance level (e.g., 0.05) stably in the tail of the empirical distribution.
#'
#' @param n_surr An integer specifying the number of surrogates requested.
#' @param sig_level A numeric value for the significance level (e.g., 0.05).
#' @param num_tests An integer specifying the number of simultaneous tests (e.g., max_lag or p*(p-1)/2).
#' @return NULL. Throws an error or warning if n_surr is insufficient.
#' @noRd
check_surrogate_count <- function(n_surr, sig_level, num_tests) {
    # 1. Minimum required just to calculate the quantile
    # Example: If sig_level = 0.05, we need at least 1/0.05 - 1 = 19 surrogates.
    min_required <- ceiling(1 / sig_level) - 1
    if (n_surr < min_required) {
        stop(sprintf(
            "Error: n_surr = %d is too small to calculate the %g%% threshold. Minimum required is %d.",
            n_surr,
            (1 - sig_level) * 100,
            min_required
        ))
    }

    # 2. Recommended number for stable max-statistic estimation in the right tail
    # The tail becomes thicker with more simultaneous tests.
    recommended <- ceiling(num_tests / sig_level)
    recommended <- max(399, recommended) # Enforce typical non-parametric baseline (e.g., 399 or 999)

    if (n_surr < recommended) {
        warning(sprintf(
            "Warning: For %d simultaneous tests at sig_level = %g, the empirical distribution of the max-statistic may be unstable with n_surr = %d. Recommended n_surr is at least %d.",
            num_tests,
            sig_level,
            n_surr,
            recommended
        ))
    }

    invisible(NULL)
}

#' Rolling Window Analysis for Xi-ACF
#'
#' @param x A numeric vector representing the time series data.
#' @param time_index An optional vector representing timestamps.
#' @param window_size An integer specifying the size of the rolling window.
#' @param step_size An integer specifying the step size. Default is 1.
#' @param max_lag An integer specifying the maximum lag to compute.
#' @param n_surr An integer specifying the number of IAAFT surrogate datasets. Default is 399.
#' @param sig_level A numeric value specifying the significance level (FWER). Default is 0.05.
#' @param n_cores An integer specifying the number of cores for parallel execution.
#' @param save_dir A character string specifying the directory path to save intermediate results.
#'
#' @return A \code{data.frame} containing the rolling window results.
#'
#' @importFrom foreach foreach
#' @importFrom doFuture registerDoFuture %dofuture%
#' @importFrom future plan multisession sequential
#' @importFrom progressr progressor with_progress
#' @importFrom dplyr bind_rows
#' @importFrom stats quantile
#' @export
run_rolling_xi_acf <- function(
    x,
    time_index = NULL,
    window_size,
    step_size = 1,
    max_lag,
    n_surr = 399,
    sig_level = 0.05,
    n_cores = NULL,
    save_dir = NULL
) {
    # --- Backward Compatibility Check for sig_level ---
    if (sig_level <= 0 || sig_level >= 1) {
        stop("Parameter 'sig_level' must be strictly between 0 and 1.")
    }
    if (sig_level > 0.5) {
        warning(
            sprintf(
                "The interpretation of 'sig_level' has changed from Confidence Level to Significance Level. Your input %g has been automatically converted to %g. Please use sig_level = %g in the future.",
                sig_level,
                1 - sig_level,
                1 - sig_level
            ),
            call. = FALSE
        )
        sig_level <- 1 - sig_level
    }

    n <- length(x)
    if (n < window_size) {
        stop("window_size must be smaller than the length of x.")
    }
    if (window_size < max_lag + 2) {
        stop("window_size must be strictly greater than max_lag + 1.")
    }

    starts <- seq(1, n - window_size + 1, by = step_size)
    num_windows <- length(starts)

    if (is.null(n_cores)) {
        n_cores <- parallel::detectCores() - 1
    }
    future::plan(future::multisession, workers = n_cores)
    on.exit(future::plan(future::sequential))

    if (!is.null(save_dir) && !dir.exists(save_dir)) {
        dir.create(save_dir, recursive = TRUE)
    }

    run_with_progress <- function() {
        p <- progressr::progressor(steps = num_windows)

        results_list <- foreach::foreach(
            i = 1:num_windows,
            .options.future = list(
                packages = c("xiacf", "stats"),
                seed = TRUE
            ),
            .errorhandling = "pass"
        ) %dofuture%
            {
                p()

                idx_start <- starts[i]
                idx_end <- idx_start + window_size - 1
                x_win <- x[idx_start:idx_end]

                # Check for zero variance
                if (stats::var(x_win) == 0) {
                    return(NULL)
                }

                # Execute C++ engine for the window
                cpp_res <- compute_xi_acf_maxstat_cpp(
                    x = as.numeric(x_win),
                    max_lag = as.integer(max_lag),
                    n_surr = as.integer(n_surr),
                    max_iter = as.integer(100)
                )

                global_threshold <- stats::quantile(
                    cpp_res$max_statistic_dist,
                    probs = 1 - sig_level,
                    names = FALSE
                )

                df_window <- data.frame(
                    Window_ID = i,
                    Lag = 1:max_lag,
                    Xi = cpp_res$xi_empirical,
                    Global_Threshold = rep(global_threshold, max_lag),
                    stringsAsFactors = FALSE
                )

                df_window$Xi_Excess <- pmax(
                    0,
                    df_window$Xi - df_window$Global_Threshold
                )

                if (!is.null(time_index)) {
                    df_window$Window_Start_Time <- time_index[idx_start]
                    df_window$Window_End_Time <- time_index[idx_end]
                }

                if (!is.null(save_dir)) {
                    saveRDS(
                        df_window,
                        file = file.path(
                            save_dir,
                            sprintf("window_%06d.rds", i)
                        )
                    )
                }

                return(df_window)
            }
        return(results_list)
    }

    new_results_list <- progressr::with_progress(run_with_progress())
    new_results_list <- Filter(is.data.frame, new_results_list)
    final_df <- dplyr::bind_rows(new_results_list)

    return(final_df)
}

#' Extract Univariate Xi-ACF from a Multivariate Xi-Matrix
#'
#' @param obj An object of class \code{xi_matrix}.
#' @param var A character string specifying the variable name to extract.
#' @param ... Additional arguments passed to xi_acf.
#'
#' @return An object of class \code{xi_acf}.
#' #' @note
#' This function performs a fresh re-calculation of the surrogate data distribution specifically for the targeted variables.
#' As a result, two differences from the original matrix output should be expected:
#' 1. **FWER Recalibration:** The global threshold will be recalibrated for the bivariate (or univariate) case, typically making it less conservative and restoring statistical power for the specific pathway.
#' 2. **Monte Carlo Variation:** Due to the randomized nature of surrogate data generation, slight numerical variations in the thresholds and confidence intervals may occur unless a random seed is explicitly set (e.g., `set.seed()`) immediately prior to calling this extraction function.
#' @export
extract_xi_acf <- function(obj, var, ...) {
    if (!inherits(obj, "xi_matrix")) {
        stop("Object must be of class 'xi_matrix'.")
    }

    raw_data_name <- "data_raw"
    if (is.null(obj[[raw_data_name]])) {
        stop(
            "Raw data is not preserved in the xi_matrix object. Cannot compute on-demand ACF."
        )
    }

    if (!(var %in% colnames(obj[[raw_data_name]]))) {
        stop(sprintf(
            "Variable '%s' not found in the original matrix data.",
            var
        ))
    }

    x_raw <- obj[[raw_data_name]][[var]]

    # Re-calculate xi_acf on demand
    res <- xi_acf(
        x = x_raw,
        max_lag = obj$max_lag,
        n_surr = obj$n_surr,
        sig_level = obj$sig_level,
        ...
    )

    # Clean up the messy deparsed variable name
    res$x_name <- var

    return(res)
}

#' Extract Bivariate Xi-CCF from a Multivariate Xi-Matrix
#'
#' @param obj An object of class \code{xi_matrix}.
#' @param var_x A character string specifying the lead variable.
#' @param var_y A character string specifying the lag variable.
#' @param ... Additional arguments passed to xi_ccf.
#'
#' @return An object of class \code{xi_ccf}.
#' #' @note
#' This function performs a fresh re-calculation of the surrogate data distribution specifically for the targeted variables.
#' As a result, two differences from the original matrix output should be expected:
#' 1. **FWER Recalibration:** The global threshold will be recalibrated for the bivariate (or univariate) case, typically making it less conservative and restoring statistical power for the specific pathway.
#' 2. **Monte Carlo Variation:** Due to the randomized nature of surrogate data generation, slight numerical variations in the thresholds and confidence intervals may occur unless a random seed is explicitly set (e.g., `set.seed()`) immediately prior to calling this extraction function.
#' @export
extract_xi_ccf <- function(obj, var_x, var_y, ...) {
    if (!inherits(obj, "xi_matrix")) {
        stop("Object must be of class 'xi_matrix'.")
    }

    raw_data_name <- "data_raw"
    if (is.null(obj[[raw_data_name]])) {
        stop(
            "Raw data is not preserved in the xi_matrix object. Cannot compute on-demand CCF."
        )
    }

    if (
        !(var_x %in% colnames(obj[[raw_data_name]])) ||
            !(var_y %in% colnames(obj[[raw_data_name]]))
    ) {
        stop(sprintf(
            "Variables '%s' and/or '%s' not found in the original matrix data.",
            var_x,
            var_y
        ))
    }

    # Re-calculate xi_ccf on demand
    res <- xi_ccf(
        x = obj[[raw_data_name]][[var_x]],
        y = obj[[raw_data_name]][[var_y]],
        max_lag = obj$max_lag,
        n_surr = obj$n_surr,
        sig_level = obj$sig_level,
        ...
    )

    # Capture the messy auto-generated names from substitute()
    old_x <- res$x_name
    old_y <- res$y_name

    # Override the metadata names
    res$x_name <- var_x
    res$y_name <- var_y

    # Clean up the data frame columns so autoplot facet labels are elegant
    if (!is.null(res$data$Lead_Var)) {
        res$data$Lead_Var[res$data$Lead_Var == old_x] <- var_x
        res$data$Lead_Var[res$data$Lead_Var == old_y] <- var_y
    }
    if (!is.null(res$data$Lag_Var)) {
        res$data$Lag_Var[res$data$Lag_Var == old_x] <- var_x
        res$data$Lag_Var[res$data$Lag_Var == old_y] <- var_y
    }

    return(res)
}

# tests/testthat/test-extractors.R

test_that("extract_xi_acf and extract_xi_ccf work correctly and clean variable names", {
    set.seed(42)
    n_test <- 50
    df_test <- data.frame(
        X = rnorm(n_test),
        Y = rnorm(n_test),
        Z = rnorm(n_test)
    )

    # Calculation of the parent xi_matrix
    res_mat <- suppressWarnings(xi_matrix(df_test, max_lag = 2, n_surr = 20))

    # 1. Test for extract_xi_acf
    ext_acf <- suppressWarnings(extract_xi_acf(res_mat, var = "X"))
    expect_s3_class(ext_acf, "xi_acf")
    # Check if variable names are cleaned up correctly
    expect_equal(ext_acf$x_name, "X")

    # 2. Test for extract_xi_ccf
    ext_ccf <- suppressWarnings(extract_xi_ccf(
        res_mat,
        var_x = "X",
        var_y = "Y",
        direction = "both"
    ))
    expect_s3_class(ext_ccf, "xi_ccf")

    # Check cleanup of variable names (metadata)
    expect_equal(ext_ccf$x_name, "X")
    expect_equal(ext_ccf$y_name, "Y")

    # Check cleanup of variable names (Lead_Var, Lag_Var) in the tidy data frame
    expect_true(all(ext_ccf$data$Lead_Var %in% c("X", "Y")))
    expect_true(all(ext_ccf$data$Lag_Var %in% c("X", "Y")))
})

test_that("extractors throw errors for invalid inputs", {
    set.seed(42)
    df_test <- data.frame(X = rnorm(30), Y = rnorm(30))
    res_mat <- suppressWarnings(xi_matrix(df_test, max_lag = 2, n_surr = 20))

    # Specify a non-existent variable name
    expect_error(extract_xi_acf(res_mat, var = "Missing"), "not found")
    expect_error(
        extract_xi_ccf(res_mat, var_x = "X", var_y = "Missing"),
        "not found"
    )

    # Pass an invalid object
    expect_error(extract_xi_acf(list(), var = "X"), "must be of class")
})

# R/deprecated.R
# Backward compatibility mappings for deprecated functions

#' @title Deprecated functions in xiacf
#' @description These functions are provided for backward compatibility with
#' older versions of xiacf and will be removed in future releases.
#' @param x A numeric vector or matrix depending on the function.
#' @param max_lag An integer specifying the maximum lag.
#' @param n_surr An integer specifying the number of surrogate datasets.
#' @param sig_level A numeric value specifying the significance or confidence level.
#' @param max_iter An integer specifying the maximum iterations.
#' @param ... Additional arguments passed to the updated functions.
#' @name xiacf-deprecated
NULL

#' @rdname xiacf-deprecated
#' @export
xi_test <- function(
    x,
    max_lag = 10,
    n_surr = 399,
    sig_level = 0.95,
    max_iter = 100,
    ...
) {
    .Deprecated("xi_acf", package = "xiacf")

    # Automatically detect and convert old confidence level (e.g., 0.95) to new significance level (e.g., 0.05)
    if (sig_level > 0.5) {
        warning(
            sprintf(
                "In legacy xi_test(), sig_level = %g is interpreted as a Confidence Level and has been automatically converted to a Significance Level (sig_level = %g) for xi_acf(). Please migrate to xi_acf(sig_level = 0.05) in the future.",
                sig_level,
                1 - sig_level
            ),
            call. = FALSE
        )
        sig_level <- 1 - sig_level
    }

    xi_acf(
        x = x,
        max_lag = max_lag,
        n_surr = n_surr,
        sig_level = sig_level,
        max_iter = max_iter,
        ...
    )
}

#' @rdname xiacf-deprecated
#' @export
generate_iaaft_surrogate <- function(...) {
    .Deprecated("surrogate_iaaft_cpp", package = "xiacf")
    surrogate_iaaft_cpp(...)
}

#' @rdname xiacf-deprecated
#' @export
generate_miaaft_surrogates <- function(...) {
    .Deprecated("surrogate_miaaft_cpp", package = "xiacf")
    surrogate_miaaft_cpp(...)
}

#' @rdname xiacf-deprecated
#' @export
generate_miaaft_surrogate_cpp <- function(...) {
    .Deprecated("surrogate_miaaft_cpp", package = "xiacf")
    surrogate_miaaft_cpp(...)
}

#' @rdname xiacf-deprecated
#' @export
run_rolling_xi_analysis <- function(...) {
    .Deprecated("run_rolling_xi_acf", package = "xiacf")
    run_rolling_xi_acf(...)
}

#' @rdname xiacf-deprecated
#' @export
compute_xi_acf_iaaft <- function(...) {
    .Deprecated("compute_xi_acf_maxstat_cpp", package = "xiacf")
    compute_xi_acf_maxstat_cpp(...)
}

#' @rdname xiacf-deprecated
#' @export
compute_xi_ccf_miaaft <- function(...) {
    .Deprecated("compute_xi_ccf_maxstat_cpp", package = "xiacf")
    compute_xi_ccf_maxstat_cpp(...)
}

#' @rdname xiacf-deprecated
#' @export
compute_xi_matrix_miaaft <- function(...) {
    .Deprecated("compute_xi_matrix_maxstat_cpp", package = "xiacf")
    compute_xi_matrix_maxstat_cpp(...)
}

# tests/testthat/test-compatibility.R

test_that("sig_level is correctly inverted for backward compatibility", {
    set.seed(42)
    x <- rnorm(100)
    y <- rnorm(100)
    df <- data.frame(A = x, B = y)

    # Specify n_surr = 400 to avoid the "too few surrogates" warning
    # 1. xi_acf check
    expect_warning(
        res_acf <- xiacf::xi_acf(
            x,
            max_lag = 2,
            n_surr = 400,
            sig_level = 0.95
        ),
        "Confidence Level to Significance Level"
    )
    expect_equal(res_acf$sig_level, 0.05)

    # 2. xi_ccf check
    expect_warning(
        res_ccf <- xiacf::xi_ccf(
            x,
            y,
            max_lag = 2,
            n_surr = 400,
            sig_level = 0.95
        ),
        "Confidence Level to Significance Level"
    )
    expect_equal(res_ccf$sig_level, 0.05)

    # 3. xi_matrix check
    expect_warning(
        res_mat <- xiacf::xi_matrix(
            df,
            max_lag = 2,
            n_surr = 400,
            sig_level = 0.95
        ),
        "Confidence Level to Significance Level"
    )
    expect_equal(res_mat$sig_level, 0.05)
})

test_that("deprecated functions issue warnings and function correctly", {
    set.seed(42)
    x <- rnorm(100)

    # xi_test issues two warnings simultaneously: "deprecated" and
    # "Confidence Level" (inverted significance level).
    # Therefore, we use a double expect_warning to capture both.
    expect_warning(
        expect_warning(
            res_test <- xi_test(x, max_lag = 2, n_surr = 400, sig_level = 0.95),
            "Confidence Level"
        ),
        "deprecated"
    )
    expect_s3_class(res_test, "xi_acf")
    expect_equal(res_test$sig_level, 0.05)

    tmp_dir <- tempfile("xi_compat_test")
    dir.create(tmp_dir)
    on.exit(unlink(tmp_dir, recursive = TRUE))

    # Remove suppressWarnings to cleanly capture the two warnings here as well
    expect_warning(
        expect_warning(
            res_roll <- run_rolling_xi_analysis(
                x = x,
                window_size = 80,
                step_size = 10,
                max_lag = 2,
                n_surr = 400,
                sig_level = 0.95,
                save_dir = tmp_dir
            ),
            "Confidence Level"
        ),
        "deprecated"
    )
    expect_s3_class(res_roll, "data.frame")
})

#' Multivariate Xi-Correlogram Matrix
#'
#' Computes the pairwise directional Chatterjee's Xi coefficient for a multivariate time series.
#'
#' @param x A numeric matrix or data.frame containing the multivariate time series (columns = variables).
#' @param max_lag An integer specifying the maximum positive lag to compute. Default is 10.
#' @param n_surr An integer specifying the number of MIAAFT surrogate datasets. Default is 399.
#' @param sig_level A numeric value between 0 and 1 specifying the significance level. Default is 0.05.
#' @param max_iter An integer specifying the maximum iterations for the MIAAFT algorithm. Default is 100.
#' @param ... Additional arguments.
#'
#' @return An S3 object of class \code{xi_matrix} containing a tidy data frame of pairwise results.
#' @importFrom stats quantile var
#' @export
xi_matrix <- function(
    x,
    max_lag = 10,
    n_surr = 399,
    sig_level = 0.05,
    max_iter = 100,
    ...
) {
    # --- Backward Compatibility Check for sig_level ---
    if (sig_level <= 0 || sig_level >= 1) {
        stop("Parameter 'sig_level' must be strictly between 0 and 1.")
    }
    if (sig_level > 0.5) {
        warning(
            sprintf(
                "The interpretation of 'sig_level' has changed from Confidence Level to Significance Level. Your input %g has been automatically converted to %g. Please use sig_level = %g in the future.",
                sig_level,
                1 - sig_level,
                1 - sig_level
            ),
            call. = FALSE
        )
        sig_level <- 1 - sig_level
    }

    if (!is.matrix(x) && !is.data.frame(x)) {
        stop("Input 'x' must be a numeric matrix or data.frame.")
    }
    x_mat <- as.matrix(x)

    if (!is.numeric(x_mat)) {
        stop("Input 'x' must be completely numeric.")
    }
    if (any(is.na(x_mat))) {
        stop(
            "Input 'x' contains NA values. Please handle them before running xi_matrix()."
        )
    }

    n <- nrow(x_mat)
    p <- ncol(x_mat)

    if (p < 2) {
        stop(
            "Input 'x' must have at least 2 columns to compute pairwise correlations."
        )
    }
    if (n < (max_lag + 2)) {
        stop(sprintf(
            "Time series length (%d) is too short for max_lag = %d.",
            n,
            max_lag
        ))
    }
    if (sig_level <= 0 || sig_level >= 1) {
        stop("Parameter 'sig_level' must be strictly between 0 and 1.")
    }

    var_names <- colnames(x)
    if (is.null(var_names)) {
        var_names <- paste0("V", 1:p)
    }

    for (j in 1:p) {
        if (stats::var(x_mat[, j]) == 0) {
            stop(sprintf(
                "Variable '%s' has zero variance. Matrix calculations cannot proceed.",
                var_names[j]
            ))
        }
    }

    num_pairs <- p * (p - 1)

    # Family A (Lag 0) has 'num_pairs' tests.
    # Family B (Lag > 0) has 'num_pairs * max_lag' tests.
    # We use the maximum family size to evaluate the required number of surrogates.
    num_tests <- num_pairs * max(1, max_lag)

    check_surrogate_count(
        n_surr = n_surr,
        sig_level = sig_level,
        num_tests = num_tests
    )

    # Execute C++ engine for the entire matrix
    cpp_res <- compute_xi_matrix_maxstat_cpp(
        X = x_mat,
        max_lag = as.integer(max_lag),
        n_surr = as.integer(n_surr),
        max_iter = as.integer(max_iter)
    )

    # Extract results and populate dataframe
    res_list <- list()
    idx <- 1

    for (i in 1:p) {
        for (j in 1:p) {
            for (k in 0:max_lag) {
                res_list[[idx]] <- data.frame(
                    Lead_Var = var_names[i],
                    Lag_Var = var_names[j],
                    Lag = k,
                    Xi = cpp_res$xi_emp[i, j, k + 1],
                    stringsAsFactors = FALSE
                )
                idx <- idx + 1
            }
        }
    }

    res_df <- dplyr::bind_rows(res_list)

    # Calculate two independent global thresholds
    threshold_lag0 <- stats::quantile(
        cpp_res$max_dist_lag0,
        probs = 1 - sig_level,
        names = FALSE,
        na.rm = TRUE
    )
    threshold_lagged <- stats::quantile(
        cpp_res$max_dist_lagged,
        probs = 1 - sig_level,
        names = FALSE,
        na.rm = TRUE
    )

    # Identify row attributes
    is_self <- res_df$Lead_Var == res_df$Lag_Var
    is_lag_zero <- res_df$Lag == 0

    # Dynamic threshold assignment (Disable for autocorrelation by setting to NA)
    res_df$Global_Threshold <- NA_real_
    res_df$Global_Threshold[!is_self & is_lag_zero] <- threshold_lag0 # Strict threshold for Lag 0 confounding
    res_df$Global_Threshold[!is_self & !is_lag_zero] <- threshold_lagged # Normal threshold for Lag > 0 propagation

    # Calculate Xi_Excess
    res_df$Xi_Excess <- NA_real_
    res_df$Xi_Excess[!is_self] <- pmax(
        0,
        res_df$Xi[!is_self] - res_df$Global_Threshold[!is_self]
    )

    # Store raw data for extractors
    df_raw <- as.data.frame(x_mat)
    colnames(df_raw) <- var_names

    out <- list(
        data = res_df,
        n = n,
        p = p,
        max_lag = max_lag,
        n_surr = n_surr,
        sig_level = sig_level,
        data_raw = df_raw
    )

    class(out) <- "xi_matrix"
    return(out)
}

#' Print method for xi_matrix
#' @param x An object of class \code{xi_matrix}.
#' @param ... Additional arguments passed to print.
#' @return The original object \code{x} invisibly.
#' @importFrom utils head
#' @export
print.xi_matrix <- function(x, ...) {
    cat("\n=== Multivariate Xi-Correlogram Matrix ===\n")
    cat(sprintf(
        "Variables: %d (%s)\n",
        x$p,
        paste(x$var_names, collapse = ", ")
    ))
    cat(sprintf("Time series length: %d\n", x$n))
    cat(sprintf("Max Lag: %d\n", x$max_lag))
    cat(sprintf("Surrogates (MIAAFT): %d\n", x$n_surr))
    cat(sprintf("Significance Level: %g (FWER controlled)\n", x$sig_level))
    cat("==========================================\n")

    sig_data <- x$data[
        which(x$data$Xi_Excess > 0),
        c("Lead_Var", "Lag_Var", "Lag", "Xi", "Global_Threshold", "Xi_Excess")
    ]

    if (nrow(sig_data) == 0) {
        cat(
            "No significant multivariate dependencies found above the global threshold.\n"
        )
    } else {
        cat("Top 5 Strongest Non-linear Causal Pathways:\n")
        print(
            utils::head(sig_data[order(-sig_data$Xi_Excess), ], 5),
            row.names = FALSE
        )
    }
    cat("\n")
    invisible(x)
}

# tests/testthat/test-xi_matrix.R

test_that("xi_matrix input validation works", {
    expect_error(xi_matrix(c(1, 2, 3)), "numeric matrix or data.frame")
    expect_error(
        xi_matrix(data.frame(A = letters[1:10], B = 1:10)),
        "completely numeric"
    )
    expect_error(
        xi_matrix(data.frame(A = c(1, NA, 3, 4, 5), B = 1:5)),
        "NA values"
    )
    expect_error(xi_matrix(matrix(rnorm(10), ncol = 1)), "at least 2 columns")
})

test_that("xi_matrix computes correctly and returns proper structure", {
    set.seed(42)
    df <- data.frame(A = rnorm(30), B = rnorm(30), C = rnorm(30))
    p <- ncol(df)
    max_lag <- 2

    res <- suppressWarnings(xi_matrix(df, max_lag = max_lag, n_surr = 20))

    expect_s3_class(res, "xi_matrix")

    # Because it includes Lag 0, the total number of rows is p * p * (max_lag + 1)
    expect_equal(nrow(res$data), p * p * (max_lag + 1))

    # Check if data_raw is saved correctly
    expect_identical(res$data_raw, df)

    # Check if threshold and Excess for diagonal elements (autocorrelation) are NA (confirming self-loop exclusion)
    diags <- res$data[res$data$Lead_Var == res$data$Lag_Var, ]
    expect_true(all(is.na(diags$Global_Threshold)))
    expect_true(all(is.na(diags$Xi_Excess)))

    # Check if threshold is calculated for off-diagonal elements (cross-correlation)
    off_diags <- res$data[res$data$Lead_Var != res$data$Lag_Var, ]
    expect_true(all(!is.na(off_diags$Global_Threshold)))
    expect_true(all(!is.na(off_diags$Xi_Excess)))
})

test_that("xi_matrix properly handles Dual-Family FWER and completely excludes autocorrelations", {
    set.seed(42)
    df_input <- data.frame(A = rnorm(50), B = rnorm(50), C = rnorm(50))

    res <- suppressWarnings(xiacf::xi_matrix(
        df_input,
        max_lag = 2,
        n_surr = 100,
        sig_level = 0.05
    ))

    df <- res$data

    # 1. Autocorrelation Exclusion (Must be NA to prevent FWER inflation)
    auto_corr <- df[df$Lead_Var == df$Lag_Var, ]
    expect_true(all(is.na(auto_corr$Global_Threshold)))
    expect_true(all(is.na(auto_corr$Xi_Excess)))

    # 2. Dual-Family Thresholds for Cross-correlation
    cross_corr <- df[df$Lead_Var != df$Lag_Var, ]
    thresh_lag0 <- cross_corr$Global_Threshold[cross_corr$Lag == 0][1]
    thresh_lagged <- cross_corr$Global_Threshold[cross_corr$Lag > 0][1]

    expect_false(is.na(thresh_lag0))
    expect_false(is.na(thresh_lagged))

    # Thresholds must be distinct due to the Dual-Family separation policy
    expect_true(thresh_lag0 != thresh_lagged)
})

#' @importFrom ggplot2 autoplot ggplot aes geom_hline geom_vline geom_ribbon geom_line geom_point scale_color_manual scale_fill_manual scale_size_manual scale_x_continuous labs theme_minimal theme element_text element_blank element_rect coord_cartesian guides guide_legend
#' @importFrom patchwork wrap_plots plot_annotation
#' @export
ggplot2::autoplot

#' Plot method for xi_acf objects
#'
#' @param object An object of class \code{xi_acf}.
#' @param ... Additional arguments passed to other methods.
#' @method autoplot xi_acf
#' @export
autoplot.xi_acf <- function(object, ...) {
    df <- object$data
    sig_pct <- object$sig_level * 100

    df$Significant <- df$Xi_Excess > 0

    p <- ggplot2::ggplot(df, ggplot2::aes(x = Lag)) +
        ggplot2::geom_ribbon(
            ggplot2::aes(ymin = 0, ymax = Global_Threshold),
            fill = "gray50",
            alpha = 0.2
        ) +
        ggplot2::geom_hline(yintercept = 0, color = "gray50", linewidth = 0.3) +
        ggplot2::geom_hline(
            yintercept = c(df$ACF_CI[1], -df$ACF_CI[1]),
            color = "blue",
            linetype = "dotted",
            linewidth = 0.4,
            alpha = 0.6
        ) +
        ggplot2::geom_line(
            ggplot2::aes(y = ACF, color = "Pearson (Linear)"),
            linetype = "dashed",
            linewidth = 0.6
        ) +
        ggplot2::geom_point(
            ggplot2::aes(y = ACF, color = "Pearson (Linear)"),
            shape = 16,
            size = 3
        ) +
        ggplot2::geom_line(
            ggplot2::aes(y = Xi, color = "Xi (Non-linear)"),
            linewidth = 0.8
        ) +
        ggplot2::geom_point(
            ggplot2::aes(
                y = Xi,
                color = "Xi (Non-linear)",
                fill = Significant,
                size = Significant
            ),
            shape = 24
        ) +
        ggplot2::scale_fill_manual(
            values = c("FALSE" = "white", "TRUE" = "firebrick"),
            guide = "none"
        ) +
        ggplot2::scale_size_manual(
            values = c("FALSE" = 1.5, "TRUE" = 3.0),
            guide = "none"
        ) +
        ggplot2::scale_color_manual(
            name = "Correlation Measure",
            values = c(
                "Xi (Non-linear)" = "firebrick",
                "Pearson (Linear)" = "steelblue"
            ),
            labels = c(
                "Xi (Non-linear)" = bquote(xi * " (Non-linear)"),
                "Pearson (Linear)" = "Pearson (Linear)"
            )
        ) +
        ggplot2::guides(
            color = ggplot2::guide_legend(
                override.aes = list(
                    shape = c(16, 24),
                    fill = c(NA, "firebrick"),
                    size = 3,
                    linetype = c("dashed", "solid")
                )
            )
        ) +
        ggplot2::scale_x_continuous(breaks = function(x) {
            seq(floor(x[1]), ceiling(x[2]), by = 1)
        }) +
        ggplot2::labs(
            title = expression(xi * "-Autocorrelation Function"),
            subtitle = bquote(
                "Gray Ribbon: FWER " * .(sig_pct) * "% Threshold"
            ),
            y = "Correlation Coefficient",
            x = "Lag"
        ) +
        ggplot2::theme_minimal() +
        ggplot2::theme(legend.position = "bottom")

    return(p)
}

#' Plot method for xi_ccf objects
#'
#' @param object An object of class \code{xi_ccf}.
#' @param ... Additional arguments passed to other methods.
#' @method autoplot xi_ccf
#' @export
autoplot.xi_ccf <- function(object, ...) {
    df <- object$data
    sig_pct <- object$sig_level * 100

    # Flag for significant pathways and create labels for faceting
    df$Significant <- df$Xi_Excess > 0
    df$Pathway <- paste(df$Lead_Var, " leads ", df$Lag_Var)

    p <- ggplot2::ggplot(df, ggplot2::aes(x = Lag)) +
        # Max-Statistic Global Threshold (Shared Ribbon)
        ggplot2::geom_ribbon(
            ggplot2::aes(ymin = 0, ymax = Global_Threshold),
            fill = "gray50",
            alpha = 0.2
        ) +
        # Zero baseline
        ggplot2::geom_hline(yintercept = 0, color = "gray50", linewidth = 0.3) +
        # Linear CCF Confidence Intervals
        ggplot2::geom_hline(
            yintercept = c(df$CCF_CI[1], -df$CCF_CI[1]),
            color = "blue",
            linetype = "dotted",
            linewidth = 0.4,
            alpha = 0.6
        ) +
        # Pearson CCF (Linear) - Line
        ggplot2::geom_line(
            ggplot2::aes(y = CCF, color = "Pearson (Linear)"),
            linetype = "dashed",
            linewidth = 0.6
        ) +
        # Pearson CCF (Linear) - Points
        ggplot2::geom_point(
            ggplot2::aes(y = CCF, color = "Pearson (Linear)"),
            shape = 16,
            size = 3
        ) +
        # Xi CCF (Non-linear) - Line
        ggplot2::geom_line(
            ggplot2::aes(y = Xi, color = "Xi (Non-linear)"),
            linewidth = 0.8
        ) +
        # Xi CCF (Non-linear) - Points (Triangles)
        ggplot2::geom_point(
            ggplot2::aes(
                y = Xi,
                color = "Xi (Non-linear)",
                fill = Significant,
                size = Significant
            ),
            shape = 24
        ) +
        # Manual fill and size scales for significance highlighting
        ggplot2::scale_fill_manual(
            values = c("FALSE" = "white", "TRUE" = "firebrick"),
            guide = "none"
        ) +
        ggplot2::scale_size_manual(
            values = c("FALSE" = 1.5, "TRUE" = 3.0),
            guide = "none"
        ) +
        # Color mapping and custom legend labels
        ggplot2::scale_color_manual(
            name = "Correlation Measure",
            values = c(
                "Xi (Non-linear)" = "firebrick",
                "Pearson (Linear)" = "steelblue"
            ),
            labels = c(
                "Xi (Non-linear)" = bquote(xi * " (Non-linear)"),
                "Pearson (Linear)" = "Pearson (Linear)"
            )
        ) +
        # Custom legend aesthetic overrides
        ggplot2::guides(
            color = ggplot2::guide_legend(
                override.aes = list(
                    shape = c(16, 24),
                    fill = c(NA, "firebrick"),
                    size = 3,
                    linetype = c("dashed", "solid")
                )
            )
        ) +
        # Integer breaks for X-axis
        ggplot2::scale_x_continuous(breaks = function(x) {
            seq(floor(x[1]), ceiling(x[2]), by = 1)
        }) +
        # Facet by causal direction (stacked vertically)
        ggplot2::facet_wrap(~Pathway, ncol = 1) +
        # Labels and titles
        ggplot2::labs(
            title = expression(xi * "-Cross-Correlation Function"),
            subtitle = bquote(
                "Gray Ribbon: FWER " * .(sig_pct) * "% Threshold"
            ),
            y = "Correlation Coefficient",
            x = "Lag"
        ) +
        ggplot2::theme_minimal() +
        ggplot2::theme(
            legend.position = "bottom",
            strip.text = ggplot2::element_text(face = "bold", size = 11)
        )

    return(p)
}

#' Plot method for xi_matrix objects
#'
#' @param object An object of class \code{xi_matrix}.
#' @param ... Additional arguments passed to other methods.
#' @method autoplot xi_matrix
#' @export
autoplot.xi_matrix <- function(object, ...) {
    df <- object$data
    sig_pct <- object$sig_level * 100

    # Identify diagonal (auto-correlation) and off-diagonal (cross-correlation) elements
    df$is_diag <- df$Lead_Var == df$Lag_Var
    df$Significant <- df$Xi_Excess > 0

    # Separate data for plotting to completely avoid NA warnings
    df_off_diag <- df[!df$is_diag, ]

    # Add a dummy column for yintercept to strictly bind hline to off-diagonal panels
    df_off_diag$zero_line <- 0

    # Calculate exact mathematical center of the Lag axis for precise text placement
    center_lag <- (max(df$Lag, na.rm = TRUE) + min(df$Lag, na.rm = TRUE)) / 2

    # Create a minimal data frame for diagonal text labels
    vars <- unique(df$Lead_Var)
    df_diag <- data.frame(
        Lead_Var = vars,
        Lag_Var = vars,
        Lag = center_lag, # Placed at the exact horizontal center
        Xi = 0.5, # Placed at the exact vertical center
        Label = vars,
        stringsAsFactors = FALSE
    )

    p <- ggplot2::ggplot(mapping = ggplot2::aes(x = Lag, y = Xi)) +
        # Off-diagonal: Max-Statistic Global Threshold (Shared Ribbon)
        ggplot2::geom_ribbon(
            data = df_off_diag,
            ggplot2::aes(ymin = 0, ymax = Global_Threshold),
            fill = "gray50",
            alpha = 0.2
        ) +
        # Off-diagonal: Zero baseline (Mapped inside aes to strictly avoid panel bleeding and warnings)
        ggplot2::geom_hline(
            data = df_off_diag,
            ggplot2::aes(yintercept = zero_line),
            color = "gray50",
            linewidth = 0.3
        ) +
        # Off-diagonal: Directional dependency lines
        ggplot2::geom_line(
            data = df_off_diag,
            color = "firebrick",
            linewidth = 0.8
        ) +
        # Off-diagonal: Highlight significant points
        ggplot2::geom_point(
            data = df_off_diag,
            ggplot2::aes(fill = Significant, size = Significant),
            color = "firebrick",
            shape = 24
        ) +
        # Manual scales for significance highlighting
        ggplot2::scale_fill_manual(
            values = c("FALSE" = "white", "TRUE" = "firebrick"),
            guide = "none"
        ) +
        ggplot2::scale_size_manual(
            values = c("FALSE" = 1.5, "TRUE" = 3.0),
            guide = "none"
        ) +
        # Diagonal: Variable names as large text
        ggplot2::geom_text(
            data = df_diag,
            ggplot2::aes(label = Label),
            size = 7,
            fontface = "bold",
            color = "black",
            hjust = 0.5, # Force strict horizontal centering
            vjust = 0.5 # Force strict vertical centering
        ) +
        # Grid layout: Lead_Var (Rows) -> Lag_Var (Columns)
        ggplot2::facet_grid(Lead_Var ~ Lag_Var) +
        # Fixed Y-axis limits
        ggplot2::coord_cartesian(ylim = c(0, 1)) +
        # Integer breaks for X-axis
        ggplot2::scale_x_continuous(breaks = function(x) {
            seq(floor(x[1]), ceiling(x[2]), by = 1)
        }) +
        ggplot2::labs(
            title = "Multivariate Xi-Correlogram Matrix",
            subtitle = bquote(
                "Gray Ribbon: Max-Statistic FWER " * .(sig_pct) * "% Threshold"
            ),
            y = expression(xi ~ ~Coefficient),
            x = "Lag"
        ) +
        ggplot2::theme_minimal() +
        ggplot2::theme(
            strip.text = ggplot2::element_text(face = "bold", size = 11),
            # Add subtle borders to make the grid structure clear
            panel.border = ggplot2::element_rect(
                fill = NA,
                color = "gray80",
                linewidth = 0.5
            )
        )

    return(p)
}

# Generated by using Rcpp::compileAttributes() -> do not edit by hand
# Generator token: 10BE3573-1514-4C36-9D1C-5A225CD40393

#' Generate Multiple IAAFT Surrogates (Univariate)
#' @param x A numeric vector.
#' @param n_surr Number of surrogates to generate.
#' @param max_iter Maximum iterations for IAAFT.
#' @return A matrix of surrogates (N x n_surr).
#' @export
surrogate_iaaft_cpp <- function(x, n_surr, max_iter = 100L) {
    .Call(`_xiacf_surrogate_iaaft_cpp`, x, n_surr, max_iter)
}

#' Generate Multiple MIAAFT Surrogates (3D Array / Cube)
#' @param X A numeric matrix (N x p).
#' @param n_surr Number of surrogates to generate.
#' @param max_iter Maximum iterations for MIAAFT.
#' @return A 3D array (arma::cube) of dimensions N x p x n_surr.
#' @export
surrogate_miaaft_cpp <- function(X, n_surr, max_iter = 100L) {
    .Call(`_xiacf_surrogate_miaaft_cpp`, X, n_surr, max_iter)
}

#' Compute empirical Xi-ACF and Max-Statistic null distribution
#' @export
#' @keywords internal
compute_xi_acf_maxstat_cpp <- function(x, max_lag, n_surr, max_iter = 100L) {
    .Call(`_xiacf_compute_xi_acf_maxstat_cpp`, x, max_lag, n_surr, max_iter)
}

#' Compute empirical Xi-CCF and Max-Statistic null distribution
#' @export
#' @keywords internal
compute_xi_ccf_maxstat_cpp <- function(x, y, max_lag, n_surr, max_iter = 100L, both_directions = TRUE) {
    .Call(`_xiacf_compute_xi_ccf_maxstat_cpp`, x, y, max_lag, n_surr, max_iter, both_directions)
}

#' Compute empirical Xi-Matrix and Max-Statistic null distribution
#' @export
#' @keywords internal
compute_xi_matrix_maxstat_cpp <- function(X, max_lag, n_surr, max_iter = 100L) {
    .Call(`_xiacf_compute_xi_matrix_maxstat_cpp`, X, max_lag, n_surr, max_iter)
}

#' Shuffle indices using R's random number generator
#' @noRd
NULL

#' Calculate ranks with random tie-breaking
#' @noRd
NULL

#' Compute Chatterjee's Xi Coefficient (Internal)
#' @noRd
NULL

#' Compute Chatterjee's Xi coefficient (Exported to R)
#' @param x A numeric vector.
#' @param y A numeric vector.
#' @return The Xi coefficient.
#' @export
xi_coefficient <- function(x, y) {
    .Call(`_xiacf_xi_coefficient_export`, x, y)
}

test_that("xi_coefficient detects linear and non-linear relationships", {
    n <- 100
    x <- 1:n
    y <- x
    xi_val <- xiacf::xi_coefficient(x, y)
    expect_gt(xi_val, 0.96)

    x_para <- seq(-10, 10, length.out = 100)
    y_para <- x_para^2
    xi_val_para <- xiacf::xi_coefficient(x_para, y_para)
    expect_gt(xi_val_para, 0.5)
})

test_that("Xi coefficient asymptotically converges to 0 for independent noise", {
    set.seed(123)
    n <- 1000
    x <- rnorm(n)
    y <- rnorm(n)
    xi_val_noise <- xiacf::xi_coefficient(x, y)

    # Due to the corrected denominator (n^2 - 1),
    # independent noise must yield a value very close to 0.
    expect_lt(abs(xi_val_noise), 0.05)
})

test_that("compute_xi_acf_maxstat_cpp throws error when max_lag is too large", {
    x_short <- rnorm(10)
    max_lag <- 20
    n_surr <- 20

    # Test strict validation 'n > max_lag + 1' in C++ engine
    expect_error(
        xiacf:::compute_xi_acf_maxstat_cpp(x_short, max_lag, n_surr, 10L),
        "strictly greater"
    )
})

# R/globals.R

utils::globalVariables(c(
    "Global_Threshold",
    "Lead_Var",
    "Lag_Var",
    "Significant",
    "ACF",
    "CCF",
    "Xi",
    "Lag",
    "i",
    "zero_line",
    "Label",
    "Pathway"
))

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/RcppExports.R
\name{surrogate_miaaft_cpp}
\alias{surrogate_miaaft_cpp}
\title{Generate Multiple MIAAFT Surrogates (3D Array / Cube)}
\usage{
surrogate_miaaft_cpp(X, n_surr, max_iter = 100L)
}
\arguments{
\item{X}{A numeric matrix (N x p).}

\item{n_surr}{Number of surrogates to generate.}

\item{max_iter}{Maximum iterations for MIAAFT.}
}
\value{
A 3D array (arma::cube) of dimensions N x p x n_surr.
}
\description{
Generate Multiple MIAAFT Surrogates (3D Array / Cube)
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/xi_ccf.R
\name{print.xi_ccf}
\alias{print.xi_ccf}
\title{Print method for xi_ccf}
\usage{
\method{print}{xi_ccf}(x, ...)
}
\arguments{
\item{x}{An object of class \code{xi_ccf}.}

\item{...}{Additional arguments passed to print.}
}
\value{
The original object \code{x} invisibly.
}
\description{
Print method for xi_ccf
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/xi_acf.R
\name{print.xi_acf}
\alias{print.xi_acf}
\title{Print method for xi_acf objects}
\usage{
\method{print}{xi_acf}(x, ...)
}
\arguments{
\item{x}{An object of class \code{"xi_acf"}.}

\item{...}{Additional arguments.}
}
\value{
Invisibly returns the original object.
}
\description{
Print method for xi_acf objects
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/RcppExports.R
\name{compute_xi_ccf_miaaft}
\alias{compute_xi_ccf_miaaft}
\title{Compute MIAAFT-based Directional Xi-CCF}
\usage{
compute_xi_ccf_miaaft(x, y, max_lag, n_surr)
}
\arguments{
\item{x}{First time series (numeric vector, potential cause)}

\item{y}{Second time series (numeric vector, potential effect)}

\item{max_lag}{Maximum positive lag to evaluate}

\item{n_surr}{Number of surrogate datasets to generate}
}
\value{
A list containing forward (X leads Y) and backward (Y leads X) Xi coefficients and surrogates.
}
\description{
Compute MIAAFT-based Directional Xi-CCF
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/xi_rolling_ccf.R
\name{run_rolling_xi_ccf}
\alias{run_rolling_xi_ccf}
\title{Rolling Directional Xi-CCF Analysis}
\usage{
run_rolling_xi_ccf(
  x,
  y,
  time_index = NULL,
  window_size,
  step_size = 1,
  max_lag,
  n_surr = 399,
  sig_level = 0.05,
  n_cores = NULL,
  save_dir = NULL
)
}
\arguments{
\item{x}{A numeric vector representing the first time series.}

\item{y}{A numeric vector representing the second time series.}

\item{time_index}{An optional vector representing timestamps.}

\item{window_size}{An integer specifying the size of the rolling window.}

\item{step_size}{An integer specifying the step size. Default is 1.}

\item{max_lag}{An integer specifying the maximum positive lag to compute.}

\item{n_surr}{An integer specifying the number of MIAAFT surrogate datasets. Default is 399.}

\item{sig_level}{A numeric value specifying the significance level (FWER). Default is 0.05.}

\item{n_cores}{An integer specifying the number of cores for parallel execution.}

\item{save_dir}{A character string specifying the directory path to save intermediate results.}
}
\value{
A \code{data.frame} containing the rolling window results.
}
\description{
Performs a rolling window analysis using Chatterjee's Xi cross-correlation to assess
the time-varying non-linear lead-lag relationship between two time series with FWER control.
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/RcppExports.R
\name{generate_miaaft_surrogate_cpp}
\alias{generate_miaaft_surrogate_cpp}
\title{Generate a Single MIAAFT Surrogate Matrix}
\usage{
generate_miaaft_surrogate_cpp(x, max_iter = 100L)
}
\arguments{
\item{x}{A numeric matrix (rows = time, cols = variables).}

\item{max_iter}{An integer specifying the maximum number of iterations.}
}
\value{
A numeric matrix representing the generated MIAAFT surrogate.
}
\description{
Generate a Single MIAAFT Surrogate Matrix
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/autoplot.xi_ccf.R
\name{autoplot.xi_ccf}
\alias{autoplot.xi_ccf}
\title{Plot Directional Xi-CCF Comparison}
\usage{
\method{autoplot}{xi_ccf}(object, ...)
}
\arguments{
\item{object}{An object of class \code{"xi_ccf"}.}

\item{...}{Additional arguments passed to other methods.}
}
\value{
A \code{ggplot} object representing the directional cross-correlogram.
}
\description{
Visualizes the comparison between the standard linear Cross-Correlation Function (CCF)
and the non-linear Chatterjee's Xi cross-correlation across two directions (X leads Y, Y leads X).
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/RcppExports.R
\name{compute_xi_ccf_maxstat_cpp}
\alias{compute_xi_ccf_maxstat_cpp}
\title{Compute empirical Xi-CCF and Max-Statistic null distribution}
\usage{
compute_xi_ccf_maxstat_cpp(
  x,
  y,
  max_lag,
  n_surr,
  max_iter = 100L,
  both_directions = TRUE
)
}
\description{
Compute empirical Xi-CCF and Max-Statistic null distribution
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/extract.R
\name{extract_xi_acf}
\alias{extract_xi_acf}
\title{Extract Univariate Xi-ACF from a Multivariate Xi-Matrix}
\usage{
extract_xi_acf(obj, var, ...)
}
\arguments{
\item{obj}{An object of class \code{xi_matrix}.}

\item{var}{A character string specifying the variable name to extract.}

\item{...}{Additional arguments passed to xi_acf.}
}
\value{
An object of class \code{xi_acf}.
#' @note
This function performs a fresh re-calculation of the surrogate data distribution specifically for the targeted variables.
As a result, two differences from the original matrix output should be expected:
1. **FWER Recalibration:** The global threshold will be recalibrated for the bivariate (or univariate) case, typically making it less conservative and restoring statistical power for the specific pathway.
2. **Monte Carlo Variation:** Due to the randomized nature of surrogate data generation, slight numerical variations in the thresholds and confidence intervals may occur unless a random seed is explicitly set (e.g., `set.seed()`) immediately prior to calling this extraction function.
}
\description{
Extract Univariate Xi-ACF from a Multivariate Xi-Matrix
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/RcppExports.R
\name{surrogate_iaaft_cpp}
\alias{surrogate_iaaft_cpp}
\title{Generate Multiple IAAFT Surrogates (Univariate)}
\usage{
surrogate_iaaft_cpp(x, n_surr, max_iter = 100L)
}
\arguments{
\item{x}{A numeric vector.}

\item{n_surr}{Number of surrogates to generate.}

\item{max_iter}{Maximum iterations for IAAFT.}
}
\value{
A matrix of surrogates (N x n_surr).
}
\description{
Generate Multiple IAAFT Surrogates (Univariate)
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/xi_acf.R
\name{xi_acf}
\alias{xi_acf}
\alias{print.xi_acf}
\title{Compute empirical Xi-ACF and its significance via IAAFT surrogates}
\usage{
xi_acf(x, max_lag = 10, n_surr = 399, sig_level = 0.05, max_iter = 100, ...)

\method{print}{xi_acf}(x, ...)
}
\arguments{
\item{x}{A numeric vector representing the time series data. Must not contain missing values (NA) or be a constant.}

\item{max_lag}{An integer specifying the maximum lag to compute. Default is 10.}

\item{n_surr}{An integer specifying the number of surrogate datasets to generate. Default is 399.}

\item{sig_level}{A numeric value between 0 and 1 specifying the significance level. Default is 0.05.}

\item{max_iter}{An integer specifying the maximum iterations for the IAAFT algorithm. Default is 100.}

\item{...}{Additional arguments (currently ignored).}
}
\value{
An object of class \code{xi_acf} containing the empirical ACF,
  pointwise thresholds, global threshold, and metadata.
}
\description{
Compute empirical Xi-ACF and its significance via IAAFT surrogates
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/RcppExports.R
\name{xi_coefficient}
\alias{xi_coefficient}
\title{Compute Chatterjee's Xi coefficient (Exported to R)}
\usage{
xi_coefficient(x, y)
}
\arguments{
\item{x}{A numeric vector.}

\item{y}{A numeric vector.}
}
\value{
The Xi coefficient.
}
\description{
Compute Chatterjee's Xi coefficient (Exported to R)
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/autoplot.xi_acf.R
\name{autoplot.xi_acf}
\alias{autoplot.xi_acf}
\title{Plot Xi-ACF Comparison}
\usage{
\method{autoplot}{xi_acf}(object, ...)
}
\arguments{
\item{object}{An object of class \code{"xi_acf"}.}

\item{...}{Additional arguments passed to other methods.}
}
\value{
A \code{ggplot} object representing the correlogram.
}
\description{
Visualizes the comparison between the standard linear Autocorrelation Function (ACF)
and the non-linear Chatterjee's Xi coefficient, including their respective significance thresholds.
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/RcppExports.R
\name{compute_xi_matrix_miaaft}
\alias{compute_xi_matrix_miaaft}
\title{Compute Pairwise Directional Xi-CCF for a Multivariate Matrix}
\usage{
compute_xi_matrix_miaaft(x, max_lag, n_surr)
}
\arguments{
\item{x}{A numeric matrix (rows = time, cols = variables).}

\item{max_lag}{An integer specifying the maximum positive lag.}

\item{n_surr}{An integer specifying the number of surrogate datasets.}
}
\value{
A list containing flat vectors for lead/lag variable indices, lags, 
        original Xi values, and a matrix of surrogate Xi values.
}
\description{
Compute Pairwise Directional Xi-CCF for a Multivariate Matrix
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/RcppExports.R
\name{compute_xi_matrix_maxstat_cpp}
\alias{compute_xi_matrix_maxstat_cpp}
\title{Compute empirical Xi-Matrix and Max-Statistic null distribution}
\usage{
compute_xi_matrix_maxstat_cpp(X, max_lag, n_surr, max_iter = 100L)
}
\description{
Compute empirical Xi-Matrix and Max-Statistic null distribution
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/deprecated.R
\name{xiacf-deprecated}
\alias{xiacf-deprecated}
\alias{xi_test}
\alias{generate_iaaft_surrogate}
\alias{generate_miaaft_surrogates}
\alias{generate_miaaft_surrogate_cpp}
\alias{run_rolling_xi_analysis}
\alias{compute_xi_acf_iaaft}
\alias{compute_xi_ccf_miaaft}
\alias{compute_xi_matrix_miaaft}
\title{Deprecated functions in xiacf}
\usage{
xi_test(x, max_lag = 10, n_surr = 399, sig_level = 0.95, max_iter = 100, ...)

generate_iaaft_surrogate(...)

generate_miaaft_surrogates(...)

generate_miaaft_surrogate_cpp(...)

run_rolling_xi_analysis(...)

compute_xi_acf_iaaft(...)

compute_xi_ccf_miaaft(...)

compute_xi_matrix_miaaft(...)
}
\arguments{
\item{x}{A numeric vector or matrix depending on the function.}

\item{max_lag}{An integer specifying the maximum lag.}

\item{n_surr}{An integer specifying the number of surrogate datasets.}

\item{sig_level}{A numeric value specifying the significance or confidence level.}

\item{max_iter}{An integer specifying the maximum iterations.}

\item{...}{Additional arguments passed to the updated functions.}
}
\description{
These functions are provided for backward compatibility with
older versions of xiacf and will be removed in future releases.
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/extract.R
\name{extract_xi_ccf}
\alias{extract_xi_ccf}
\title{Extract Individual Xi-CCF from Xi-Matrix}
\usage{
extract_xi_ccf(object, var_x, var_y, x_raw = NULL)
}
\arguments{
\item{object}{An object of class \code{xi_matrix}.}

\item{var_x}{A character string for the first variable (X).}

\item{var_y}{A character string for the second variable (Y).}

\item{x_raw}{Optional. The original multivariate data used to compute the matrix.}
}
\value{
An object of class \code{xi_ccf}.
}
\description{
Extracts the cross-correlation results for a specific pair of variables from
a \code{xi_matrix} object and converts it into a \code{xi_ccf} S3 object.
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/xi_ccf.R
\name{xi_ccf}
\alias{xi_ccf}
\title{Bivariate Xi-Cross-Correlation Function}
\usage{
xi_ccf(
  x,
  y,
  max_lag = 20,
  n_surr = 399,
  sig_level = 0.05,
  max_iter = 100,
  direction = c("both", "x_leads"),
  ...
)
}
\arguments{
\item{x}{A numeric vector representing the first time series.}

\item{y}{A numeric vector representing the second time series.}

\item{max_lag}{An integer specifying the maximum lag to compute. Default is 20.}

\item{n_surr}{An integer specifying the number of MIAAFT surrogate datasets. Default is 399.}

\item{sig_level}{A numeric value specifying the significance level (FWER). Default is 0.05.}

\item{max_iter}{An integer specifying the maximum iterations for the MIAAFT algorithm. Default is 100.}

\item{direction}{A character string specifying the testing direction.
"both" computes X->Y and Y->X. "x_leads" computes only X->Y. Default is "both".}

\item{...}{Additional arguments.}
}
\value{
An S3 object of class \code{xi_ccf}.
}
\description{
Computes the directional Chatterjee's Xi coefficient between two time series
across multiple lags, with Family-Wise Error Rate (FWER) control.
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/xi_matrix.R
\name{xi_matrix}
\alias{xi_matrix}
\title{Multivariate Xi-Correlogram Matrix}
\usage{
xi_matrix(x, max_lag = 10, n_surr = 399, sig_level = 0.05, max_iter = 100, ...)
}
\arguments{
\item{x}{A numeric matrix or data.frame containing the multivariate time series (columns = variables).}

\item{max_lag}{An integer specifying the maximum positive lag to compute. Default is 10.}

\item{n_surr}{An integer specifying the number of MIAAFT surrogate datasets. Default is 399.}

\item{sig_level}{A numeric value between 0 and 1 specifying the significance level. Default is 0.05.}

\item{max_iter}{An integer specifying the maximum iterations for the MIAAFT algorithm. Default is 100.}

\item{...}{Additional arguments.}
}
\value{
An S3 object of class \code{xi_matrix} containing a tidy data frame of pairwise results.
}
\description{
Computes the pairwise directional Chatterjee's Xi coefficient for a multivariate time series.
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/RcppExports.R
\name{compute_xi_acf_iaaft}
\alias{compute_xi_acf_iaaft}
\title{Compute Xi-ACF for Multiple Lags (Core Engine)}
\usage{
compute_xi_acf_iaaft(x, max_lag, n_surr)
}
\arguments{
\item{x}{A numeric vector (time series).}

\item{max_lag}{An integer specifying the maximum number of lags to compute.}

\item{n_surr}{An integer specifying the number of surrogate datasets to generate.}
}
\value{
A list containing \code{xi_original} (the Xi coefficients for the original series) 
and \code{xi_surrogates} (a matrix of Xi coefficients for the surrogate datasets).
}
\description{
Calculates Chatterjee's Xi coefficient for multiple lags and generates
IAAFT (Iterative Amplitude Adjusted Fourier Transform) surrogates to establish 
confidence intervals for non-linear dependence.
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/autoplot.R
\name{autoplot.xi_matrix}
\alias{autoplot.xi_matrix}
\title{Plot method for xi_matrix objects}
\usage{
\method{autoplot}{xi_matrix}(object, ...)
}
\arguments{
\item{object}{An object of class \code{xi_matrix}.}

\item{...}{Additional arguments passed to other methods.}
}
\description{
Plot method for xi_matrix objects
}
